<?php
/**
 * NAVT Word Press Plugin
 * Copyright (c) 2006-2007 Greg A. Bellucci/Atalaya Studio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * @package NAVT Word Press Plugin
 * @author Greg A. Bellucci <greg[AT]gbellucci[DOT]us
 * @copyright Copyright &copy; 2006-2007 Greg A. Bellucci
 * @license http://www.opensource.org/licenses/mit-license.php The MIT License
 */
require('navt_widget.php'); // includes the widget code @since 95.37
global $navt_map;

/**
 * @name NAVTdbg
 * @abstract Queried for debug on/off
 * @return boolean 1 to turn on debugging, 0 for off
 * @since .2
 */
function NAVTdbg() {
    return(isset($_POST['navtdebugon']));
}

/**
 * Navigation plugin class
 *
 * @package NAVT
 * @since created
 */
class NAVT {

    /**
    * plugin init
    *
    */
    function init() {
        global $navt_map;
        $navt_map = array();

        if(NAVTdbg()) navt_write_debug(NAVT_INIT, sprintf("\n\n[plugin] %s\n", __FUNCTION__));

        // first things first...
        // Run a backup?
        if ( isset( $_REQUEST['NAVT_backup_plugin'] ) ) {
            // initiate a configuration backup
            if(NAVTdbg()) navt_write_debug(NAVT_INIT, sprintf("Initializing backup\n"));
            unset($_POST['NAVT_backup_plugin']);
            NAVT::backup(); // this call does not return
        }

        // Run restore?
        elseif ( isset( $_REQUEST['NAVT_restore_plugin'] ) ) {
            unset($_POST['NAVT_restore_plugin']);
            $plugin = NAVT::get_url();
            $path = pathinfo($plugin);
            $new_request = $_SERVER['PHP_SELF'] . '?page=' . $path['basename'] . '/app/navt_restore.php';
            wp_redirect($new_request);
            die();
        }

        // Reset the plugin?
        elseif( isset($_POST['NAVT_uninstall_plugin'])) {
            NAVT::uninstall();
            wp_cache_flush();
            unset($_POST['NAVT_uninstall_plugin']);
        }

        // Initialize the plugin
        NAVT::register_scripts();
        NAVT::install_check();


        // determine which pages is loading
        $uri = $_SERVER['REQUEST_URI'];
        $pathinfo = pathinfo($uri);

        if( $pathinfo['basename'] == 'navt.php' ) {
            // add headers for navt list page
            add_action('admin_head', array('NAVT', 'navt_list_page'));
        }
        else {
            // add headers for everything else
            add_action('admin_head', array('NAVT', 'admin_head'));
        }

        // set wordpress action and filter callbacks
        // post hooks
        add_action('save_post', array('NAVT', 'save_post_wpcb'), 10, 2);
        add_action('delete_post', array('NAVT', 'delete_post_wpcb'), 10, 1);
        add_action('dbx_page_sidebar', array('NAVT', 'dbx_page_sidebar_wpcb'), 10, 1); // @since 95.30

        // user hooks
        add_action('edit_user_profile', array('NAVT', 'edit_user_profile_wpcb'), 10);
        add_action('profile_update', array('NAVT', 'profile_update_wpcb'), 10);
        add_action('user_register', array('NAVT', 'user_register_wpcb'), 10);
        add_action('delete_user', array('NAVT', 'delete_user_wpcb', 10, 1));

        // category hooks
        add_action('edit_category_form', array('NAVT', 'edit_category_form_wpcb'), 10, 1); // @since 95.30
        add_action('edit_category', array('NAVT', 'edit_category_wpcb'), 10, 1); // @since 95.30
        add_action('delete_category', array('NAVT', 'delete_category_wpcb'), 10, 1);
        add_filter('wp_redirect', array('NAVT', 'wp_redirect_wpcb'), 10, 2);
        add_action("created_category", array('NAVT', 'add_category_wpcb')); // @since 95.38
        add_action("edited_category", array('NAVT',  'edited_category_wpcb')); // @since 95.38

        if(!is_admin() ) {
            add_filter('list_cats_exclusions', array('NAVT', 'list_cats_exclusions_wpcb'), 10, 1);
        }

        // Content filter for handling NAVT lists inside posts or pages
        add_filter('the_content', array('NAVT','the_content_wpcb'), 10, 1);

        /**
          * Give WordPress the menu callback for this plugin
          */
        add_action('admin_menu', array('NAVT', 'admin_menu_wpcb'));

    }// end function


    /**
     * Checks NAVT installation
     */
    function install_check() {

        // get the timestamp on this file and the last one we saved
        $dir_timestamp = filemtime(dirname(__FILE__));
        $last_modified = NAVT::get_option(LASTMODIFIED);

        if(empty($last_modified) || $last_modified === false) {
            $m = NAVT::load_map();

            // Check for a map
            if( empty($m) || ($m === false) || (!is_array($m)) ) {

                // Remove any residual options that may be lying around
                // this is an installation
                NAVT::uninstall();

                // build a new map
                $m = NAVT::build_new_map();

                if(NAVTdbg()) navt_write_debug(NAVT_INIT, (sprintf("[plugin] adding options %s\n", __FUNCTION__)));
                NAVT::add_option(SCHEME, '1', SCRIPTNAME.' next scheme');
                NAVT::add_option(VERSIONID, MAJVER.MINVER, 'installed version');
                NAVT::add_option(VERCHECK, 0, 'automatic version checking (Off by default)');
                NAVT::add_option(DEF_ADD_GROUP, ID_DEFAULT_GROUP, SCRIPTNAME.' default navigation group.');
                NAVT::add_option(SERIALIZED_MAP, $m, SCRIPTNAME.' mapping structure.');
                NAVT::add_option(LASTMODIFIED, $dir_timestamp, 'Used by '.SCRIPTNAME.' for version tracking.');
                NAVT::add_option(INSTALLED, 1, SCRIPTNAME . ' installation status');
            }
        }
        elseif( $last_modified < $dir_timestamp ) {
            // just update the timestamp
            NAVT::update_option(LASTMODIFIED, $dir_timestamp, 'last updated');
        }

        // upgrade conversions and version checking
        $this_version  = intval( MAJVER.MINVER );
        $installed_ver = NAVT::get_installed_version();
        if( $installed_ver < $this_version ) {
            // convert the database info if necessary and update the version id
            NAVT::data_conversion($installed_ver);
        }
    }

    // Conversions from one release to another (if necessary)
    function data_conversion($installed_version, $this_version ='' ) {

        if( empty($installed_version) || '' == $installed_version ) {
            $installed_version = 1;
        }
        else {
            $ver = intval($installed_version); // make sure this is an integer
        }

        // For anything prior to .9542
        if( $ver < 9542 ) {

            // .95.30 adds ext field
            // .95.42 adds ex2 field
            if(NAVTdbg()) navt_write_debug(NAVT_INIT, (sprintf("[plugin] data conversion %s -> %s \n", $ver, SCRIPTVERS)));

            $new_map = array();
            $old_map = NAVT::load_map();

            for( $idx = 0; $idx < count($old_map); $idx++ ) {
                $ttl = $old_map[$idx][TTL];
                $nme = $old_map[$idx][NME];
                $ttl = str_replace('"', "'", $ttl);     // convert double quotes to single
                $nme = ('' == $nme) ? $ttl: $nme;       // make sure this is not empty
                $nme = str_replace('"', "'", $nme);     // convert double quotes to single
                $grp = strtoupper($old_map[$idx][GRP]); // forces names to upper case

                $args = array(
                'grp' => $grp,
                'typ' => $old_map[$idx][TYP],
                'idn' => $old_map[$idx][IDN],
                'ttl' => $ttl, 'nme' => $nme,
                'opt' => $old_map[$idx][OPT],
                'ext' => $old_map[$idx][EXT],
                'lvl' => '0',
                'ex2' => '');
                $new_map[] = navt_format_menu_item($args);
            }

            // Update the map and the installed version id
            NAVT::update_option(SERIALIZED_MAP, $new_map);
        }

        // update the version number
        NAVT::update_option(VERSIONID, MAJVER.MINVER);
        return;
    }

    /**
     * Returns the version last saved in the data options
     *
     * @return integer
     */
    function get_installed_version() {
        $tmp = str_replace('.','', NAVT::get_option(VERSIONID)); // in case there are 'dots' in a previous version
        $ver = intval($tmp);
        return($ver);
    }

    /**
     * Uninstalls the plugin - removes the plugin options from the database
     */
    function uninstall() {
        if(NAVTdbg()) navt_write_debug(NAVT_INIT, (sprintf("[plugin] removing options %s\n", __FUNCTION__)));
        NAVT::delete_option(SERIALIZED_MAP);
        NAVT::delete_option(LASTMODIFIED);
        NAVT::delete_option(SCHEME);
        NAVT::delete_option(VERSIONID);
        NAVT::delete_option(VERCHECK);
        NAVT::delete_option(DEF_ADD_GROUP);
        NAVT::delete_option(INSTALLED);
        NAVT::delete_option(TIPS);
    }// end function

    /**
     * register scripts used by this plugin
     */
    function register_scripts() {
        $url = NAVT::get_url();

        //wp_register_script('navt_jquery',     $url . '/js/libs/jquery.js', array(), '1.2.1');
        //wp_register_script('navt_prototype',  $url . 'js/libs/prototype.js', array(), '1.5.1');

        wp_register_script('navt_tips',
        $url . '/js/ntip/js/navtjtip.js',
        array(), VER_NAVT);

        wp_register_script('navt_options_js',
        $url . '/js/navtoptions.js.php',
        array('jquery'), VER_NAVT);

        wp_register_script('navt_js',
        $url . '/js/navt.js.php',
        array('prototype', 'jquery', 'scriptaculous-dragdrop', 'navt_tips' ), VER_NAVT);
    }
    /**
     * Registered callback function - adds the plugin options to the Management submenu
     *
     * @see Word Press function add_action('admin_menu')
     * @since 95.40 - moved plugin admin from plugin page to the bottom of the list page
     * this is because WordPress MU can block plugin page access by users.
     *
     */
    function admin_menu_wpcb() {

        // Add this menu item to the "Manage" menu to manage menus
        if ( function_exists('add_management_page') ) {
            // requires admin privedges
            add_management_page('Menu Management', 'NAVT Lists', 8, __FILE__, array('NAVT', 'configure'));
        }
    }// end function

    /**
     * Registered callback function - adds the plugin options to the Management submenu
     *
     * @see Word Press function add_action('admin_menu')
     * @since 95.38
     *
     */
    function add_alt_menu() {

        // Add this menu item to the "Manage" menu to manage menus
        if ( function_exists('add_management_page') ) {
            // requires admin privedges
            add_management_page('Menu Management', 'NAVT Lists', 8, __FILE__, array('NAVT', 'plugin_error'));
        }

    }// end function

    /**
     * Registered Word Press callback that displays plugins options form
     */
    function configure() {
        // includes the option-form script
        load_plugin_textdomain('navt_domain');
        include(NAVT_PLUGINPATH . '/display/navt-display.php');

    }// end function

    /**
     * Registered WordPress callback that adds css and script information to the admin page header.
	 */
    function navt_list_page() {
        wp_print_scripts(array('navt_js')); ?>
<script type="text/javascript">
//<![CDATA[
var navtpath = '<?php navt_output_url();?>';
addLoadEvent(navt);
function navt() {
    jQuery.noConflict();
    createSortables();
    set_view();
}
//]]>
</script>
<link type="text/css" rel="stylesheet" href="<?php navt_output_url(); ?>/css/navt.css.php" media="screen"/>
<link type="text/css" rel="stylesheet" href="<?php navt_output_url(); ?>/js/ntip/css/navtjtip.css" media="screen"/>
    <?php
    }// end function

    function admin_head() {
        wp_print_scripts(array('navt_options_js')); ?>
	   <?php
    }

    /**
     * Computes this plugin's url
     *
     * @return string URL
     */
    function get_url() {
        static $plugin_url;
        if(!isset($plugin_url)) {
            $abspath = (DIRECTORY_SEPARATOR != '/') ? str_replace(DIRECTORY_SEPARATOR, '/', ABSPATH) : ABSPATH;
            $plugin_url = str_replace($abspath, get_option( 'siteurl' ).'/', NAVT_PLUGINPATH);
        }
        return $plugin_url;
    }// end function

    /**
     * Loads the menu map from the database and saves it to a global array of variables
     *
     * @return array menu map
     */
    function load_map() {
        global $navt_map;
        if(empty($navt_map) || !count($navt_map)) {
            $navt_map = NAVT::get_option(SERIALIZED_MAP);
        }
        return($navt_map);
    }

    /**
     =================================
     * @subpackage Mapping Functions
     * @since created
     =================================
     */

    /**
     * Builds the default menu map from the currently defined Pages and Categories.
     *
     * @return array menu map
     */
    function build_new_map() {

        global $wpdb;

        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("Creating new data configuration\n"));

        // Home/Login links
        $map_array = array();

        $args = array('grp' => ID_DEFAULT_GROUP,
        'typ' => TYPE_LINK,
        'idn' => HOMEIDN,
        'ttl' => HOMETXT,
        'nme' => HOMETXT,
        'opt' => '0',
        'ext' => '',
        'lvl' => '0',
        'ex2' => ''
        );

        $map_array[] = navt_format_menu_item($args);

        $args['idn'] = LOGINIDN;
        $args['ttl'] = $args['nme'] = LOGINTXT;
        $map_array[] = navt_format_menu_item($args);

        // Build a new map from the database information
        $all_page_ids = get_all_page_ids(); //  @since 95.30
        $all_cat_ids  = get_all_category_ids(); // @since 95.30

        // Create the controls for each author
        // @since 95.38
        $users = get_users_of_blog();
        foreach($users as $user) {
            $userdata = get_userdata($user->user_id);
            $level = intval(get_usermeta($user->user_id, $wpdb->prefix . 'user_level'));
            $avatar = NAVT::get_url() . '/' . IMG_AVATAR;
            $username = '' != $userdata->first_name && '' != $userdata->last_name ?
            $userdata->first_name . ' ' . $userdata->last_name : $userdata->user_nicename;
            $ttl = str_replace('"', "'", $username);
            $args['ttl'] = $args['nme'] = $ttl;

            // Set options
            $args['opt'] = (SHOW_AVATAR | USE_DEF_AVATAR | (($level > 1) ? 0: DO_NOT_DISPLAY));
            $args['idn'] = $user->user_id;
            $args['typ'] = TYPE_AUTHOR;
            $args['lvl'] = '0';
            $args['ext'] = $avatar;
            $args['ex2'] = '';
            $map_array[] = navt_format_menu_item($args);
        }

        // Create the controls for each static page
        if($all_page_ids) {
            foreach($all_page_ids as $id) {
                $post = get_post($id);

                if(is_object($post) && $post->post_type == 'page') {
                    $args['idn'] = $id;
                    $args['typ'] = TYPE_PAGE;
                    $args['opt'] = (($post->post_status == 'draft') ? ISDRAFTPAGE: '0');
                    $args['ttl'] = str_replace('"', "'", $post->post_title);
                    $args['nme'] = $args['ttl'];
                    $args['lvl'] = NAVT::get_page_level($id);
                    $args['ext'] = $args['ex2'] = '';
                    $map_array[] = navt_format_menu_item($args);
                }
            }
        }// end if

        // Create the controls for each category
        if($all_cat_ids) {
            foreach($all_cat_ids as $id) {
                $cat = get_category($id); // @since 95.30
                $args['lvl'] = NAVT::get_category_level($id);
                $args['ttl'] = str_replace('"', "'", $cat->cat_name);
                $args['nme']= $args['ttl'];
                $args['idn'] = $cat->cat_ID;
                $args['typ'] = TYPE_CAT;
                $args['opt'] = '0';
                $args['ext'] = $args['ex2'] = '';
                $map_array[] = navt_format_menu_item($args);
            }
        }// end if

        return($map_array);

    }// end function


    /**
     * Determines what categories to exclude
     *
     * @param array - current menu map
     * @return array -  returns an array of categories to exclude
	  */
    function get_exclusions( $map_array ) {
        $exc_array = array();
        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s\n", __FUNCTION__));

        if( is_array($map_array) ) {

            foreach($map_array as $itm) {
                if( $itm[GRP] == ID_DEFAULT_GROUP ) {
                    continue;
                }
                // category ?
                if( $itm[TYP] == TYPE_CAT ) {

                    $opts = (intval($itm[OPT]));
                    $id = $itm[IDN];

                    $user_is_logged_in = is_user_logged_in();
                    $is_private = ($opts & ISPRIVATE);
                    $show_in_list = ($opts & SHOW_IN_LIST);

                    $exclude_this = 1; // normally excluded
                    $exclude_this = ($show_in_list) ? 0: $exclude_this;
                    $exclude_this = ($is_private && !$user_is_logged_in ) ? 1: $exclude_this;

                    if( !$exclude_this ) {
                        $category = get_category($id);
                        $parent = $category->category_parent;
                        if( $parent != 0 ) {
                            $exclude_this = (in_array($parent, $exc_array)) ? 1: 0;
                        }
                    }

                    if( ($exclude_this) && (!in_array($id, $exc_array)) ) {
                        $exc_array[] = $id;
                        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("\texcluding category '%s'\n: ", $itm[TTL]));
                    }
                    else {
                        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("\tallowing category '%s'\n: ", $itm[TTL]));
                    }
                }
            }// end for
        }// end if

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("\texclusion array\n: "), $exc_array);
        return( $exc_array );

    }// end function

    /**
     * Returns an array containing the group names defined in the menu map
     *
     * @param array $map_array - menu map
     * @return array - array of groups
     */
    function get_groups($map_array) {
        $grps = array();
        if( is_array($map_array) ) {
            $grps[] = ID_DEFAULT_GROUP; // must always there...
            foreach($map_array as $itm) {
                if( !in_array($itm[GRP], $grps)) {
                    $grps[] = $itm[GRP];
                }
            }
        }
        return($grps);
    }

    /**
     ===================================
     * @subpackage Word Press Callbacks
     * @since created
     ===================================
     */

    /**
     * Examines the content of a page or post
     * WordPress callback function (@see Word Press add_action('the_content')
     *
     * @param string $content - post or page content
     * @since 95.38
     */
    function the_content_wpcb($content) {

        // syntax: [NAVT group=groupname, title=heading title] ]
        // , title= is optional
        $pattern = '/\[\s*NAVT\s*group\s*=\s*[a-z0-9]+\s*(\]|\s*,\s*title\s*=.+\])/im';
        $matched = array();

        if(! preg_match_all($pattern, $content, $matched, PREG_SET_ORDER)) {
            return $content;
        }
        else {
            $replacement = array();
            foreach( $matched as $idx => $match ) {
                foreach( $match as $nidx => $value ) {
                    if( $nidx == 0 ) {
                        // save the content to be replaced
                        $replacement[$idx]['content_string'] = $value;
                        $m = array();

                        $pattern = '/\s*group\s*=\s*[a-z0-9]+(\s*|,|\])/i'; // group=
                        if( preg_match($pattern, $value, $m) ) {
                            // isolate the group name
                            $str = preg_replace('/\s+/', '',  $m[0]);     // strip spaces
                            $group = preg_replace('/group=/' , '', $str); // remove 'group='
                        }
                    }
                    else {
                        // *may* contain a title - may not
                        $title = '';
                        $pattern = '/,\s*title\s*=.+/i';
                        if( preg_match($pattern, $value, $m) ) {
                            // isolate the title
                            $str = preg_replace('/,\s*title\s*=\s*/i', '', $m[0]); // remove 'title='
                            $str = preg_replace('/\]/i', '', $str); // remove the last ']'
                            $title = trim($str); // trim it on both ends
                        }
                    }
                }
                // save these
                $replacement[$idx]['group'] = $group;
                $replacement[$idx]['title'] = $title;
            }

            $new_content = $content;
            foreach( $replacement as $idx => $arr ) {
                // get the navigation group and replace the post/page contents
                $text = navt_getlist($arr['group'], 0, $arr['title']);
                $text = "<div class='navt_content_list'>" . $text . "</div>";
                $new_content = str_replace($arr['content_string'], $text, $new_content);
            }
            $content = $new_content;
        }
        return($content);
    }

    /**
     * Add/update a page
     * This routine is registered with Word Press as an action (@see Word Press add_action('save_post')
     *
     * @param integer $id Database identifier for the page being saved
	 */
    function save_post_wpcb($id) {

        // Get this post
        if( '' != $id ) {

            $post = get_post($id);
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s id = %s\n", __FUNCTION__, $id));

            if(is_object($post) && $post->post_type == 'page') {

                // defaults
                $ttl = $nme = str_replace('"', "'", $post->post_title);
                $lvl = NAVT::get_page_level($post->ID);
                $grp = ID_DEFAULT_GROUP;
                $ext = '';
                $opt = intval( (('draft' == $post->post_status ) ? ISDRAFTPAGE: 0) );

                if( isset($_POST['NAVT']) ) {

                    foreach($_POST['NAVT'] as $option => $value) {
                        switch( $option ) {
                            case 'navgroup':    {$grp = $value;                        break;}
                            case 'linkoptions': {$opt |= intval($value);               break;}
                            case 'imgclass':    {$ext = sanitize_title($value);        break;}
                            case 'pagealias':   {$nme = (!isBlank($value)) ? $value: $ttl; break;}
                            case 'level':       {$lvl = $value;                        break;}
                            case 'isprivate':   {$opt |= ISPRIVATE;                    break;}
                            case 'nofollow':    {$opt |= NOFOLLOW;                     break;}
                            case 'donotuse':    {$opt |= DO_NOT_DISPLAY;               break;}
                        }// end switch
                    }// end foreach

                    if( isBlank($nme)) {
                        $nme = $ttl;
                    }

                    if( !isBlank($nme)) {
                        $nme = strip_tags($nme);
                        $nme = stripcslashes($nme);
                        $nme = str_replace('"', "'", $nme);
                    }

                    // Fix/correct link options
                    if( ($opt & TEXT_WITH_SIDE_GRAPHIC)  && isBlank($ext) )  {
                        $opt ^= TEXT_WITH_SIDE_GRAPHIC; // turn this off
                    }
                }

                // Is this a new page or an update?
                $map_array = NAVT::load_map();
                $idx = NAVT::get_item_index(TYPE_PAGE, $id, $map_array);

                if( $idx != (-1) ) {
                    // update
                    $map_array[$idx][GRP] = $grp;
                    $map_array[$idx][TTL] = $ttl;
                    $map_array[$idx][NME] = $nme;
                    $map_array[$idx][OPT] = $opt;
                    $map_array[$idx][EXT] = $ext;
                    $map_array[$idx][LVL] = $lvl;
                    if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s page updated\n", __FUNCTION__), $$map_array[$idx]);
                }
                else {
                    // new item
                    $args = array(  'grp' => $grp,  'typ' => TYPE_PAGE,
                    'idn' => $id,   'ttl' => $ttl,
                    'nme' => $nme,  'opt' => $opt,
                    'ext' => $ext,  'lvl' => $lvl,
                    'ex2' => '');

                    $map_array[] = navt_format_menu_item($args);
                    if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s page added\n", __FUNCTION__), $args);
                }

                // update the map
                NAVT::update_option(SERIALIZED_MAP, $map_array);
            }
        }
    }

    /**
     * Wordpress hook to add ny 2cents to the 'write' page form
     * this adds NAVT options to the sidebar area
     * This routine is registered with Word Press as an action (@see Word Press add_action('dbx_page_sidebar')
     * @since 95.30
     */
    function dbx_page_sidebar_wpcb() {

        $itm = null;

        if( isset($_GET['post']) ) {
            $post_id = $_GET['post'];
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("\tpost: %s\n", $post_id));
            $itm = NAVT::find_by_id(TYPE_PAGE, $post_id);
            if( !is_array($itm) ) unset($itm);
        }

        $grp = ((is_array($itm) && !empty($itm[GRP])) ? $itm[GRP]: ID_DEFAULT_GROUP);
        $nme = ((is_array($itm) && !empty($itm[NME])) ? $itm[NME]: '');
        $opt = ((is_array($itm) && !empty($itm[OPT])) ? $itm[OPT]: 0);
        $lvl = ((is_array($itm) && !empty($itm[LVL])) ? $itm[LVL]: 0);
        $ext = ((is_array($itm) && !empty($itm[EXT])) ? $itm[EXT]: '');
        $ex2 = ((is_array($itm) && !empty($itm[EX2])) ? $itm[EX2]: '');
        $ddlist = NAVT::build_group_dropdown($grp);
        $opt = intval($opt);

        // Added to the sidebar area of the page
        $anchor_type = ($opt & (STANDARD_LINK | TEXT_OVER_GRAPHIC | TEXT_WITH_SIDE_GRAPHIC | GRAPHIC_LINK));
 ?>
 <!-- <?php echo SCRIPTNAME . ' v' . SCRIPTVERS; ?> -->
 <script type='text/javascript'>
 /* <![CDATA[ */
 addLoadEvent(navt_page_options);
 function navt_page_options() {
     jQuery.noConflict();
     init_navt_page_options();
 }
 /* ]]> */
</script>
 <fieldset id="navtdiv" class="dbx-box">
    <h3 class="dbx-handle"> NAVT Page Options </h3>
    <div class="dbx-content">
        <p><label> Navigation Group <br /><?php echo $ddlist; ?></label></p>
        <p><label>Anchor Display Option<br />
        <select id="navt_anchor_display_option_page" name="NAVT[linkoptions]">
            <option value="<?php echo STANDARD_LINK;?>" <?php selected(STANDARD_LINK, $anchor_type);?>>Standard Text Link</option>
            <option value="<?php echo TEXT_OVER_GRAPHIC;?>" <?php selected(TEXT_OVER_GRAPHIC, $anchor_type);?>>Text over graphic</option>
            <option value="<?php echo TEXT_WITH_SIDE_GRAPHIC;?>" <?php selected(TEXT_WITH_SIDE_GRAPHIC, $anchor_type);?>>Text with side graphic</option>
            <option value="<?php echo GRAPHIC_LINK;?>" <?php selected(GRAPHIC_LINK, $anchor_type);?>>Graphic link</option>
        </select></label></p>
        <p><label id="navt_imgclass_lbl">Side graphic CSS Class<br />
        <input id="navt_imgclass_tbx" name="NAVT[imgclass]" type="text" size="20" maxlength="50" value="<?php echo $ext;?>" style="padding:3px;" /></label></p>
        <p><label>Alias Name<br />
        <input id="navt_item_alias" name="NAVT[pagealias]" type="text" size="20" maxlength="50" value="<?php echo $nme;?>" style="padding:3px;"/></label><br /></p>

        <p><label>
        <select name="NAVT[level]" style="width:8em;">
            <option value="0" <?php selected('0', $lvl);?>>Top</option>
            <option value="1" <?php selected('1', $lvl);?>>&mdash; 1</option>
            <option value="2" <?php selected('2', $lvl);?>>&mdash;&mdash; 2</option>
            <option value="3" <?php selected('3', $lvl);?>>&mdash;&mdash;&mdash; 3</option>
            <option value="4" <?php selected('4', $lvl);?>>&mdash;&mdash;&mdash;&mdash; 4</option>
        </select> Level</label><br />

        <label><input name="NAVT[isprivate]" type="checkbox" value="1" <?php checked(ISPRIVATE, ($opt & ISPRIVATE));?> /> Private</label><br />
        <label><input name="NAVT[nofollow]"  type="checkbox" value="1" <?php checked(NOFOLLOW, ($opt & NOFOLLOW));?> /> rel=nofollow</label><br />
        <label><input name="NAVT[donotuse]"  type="checkbox" value="1" <?php checked(DO_NOT_DISPLAY, ($opt & DO_NOT_DISPLAY));?> /> Do not use</label></p>
    </div>
    </fieldset>

 <!-- end -->
 <?php

 /* NOTE: submit takes us to save_post_wpcb */

    }// end function

    /**
     * Remove a static page from the menu map
     * This routine is registered with Word Press as an action (@see Word Press add_action('delete_post')
     *
     * @param integer $id Database identifier for the page being removed
	 */
    function delete_post_wpcb($id) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s arg:%s\n", __FUNCTION__, $id));

        $new_map = array();
        $was_removed = 0;
        $map_array = NAVT::load_map();

        foreach( $map_array as $itm) {

            if( ($itm[TYP] == TYPE_PAGE) && ($itm[IDN] == $id) ) {
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("\t%s: %s removed from map\n", __FUNCTION__, $id));
                $was_removed = 1;
            }
            else{
                // copy this item into the new map
                $new_map[] = $itm;
            }
        }// end for

        if( $was_removed ) {
            NAVT::update_option(SERIALIZED_MAP, $new_map);
        }

    }// end function

    /**
     * Add a new category to the menu map
     * This routine is registered with Word Press as an action (@see Word Press add_action('add_category')
     *
     * @param integer $id Database identifier for the added category
	 */
    function add_category_wpcb($cat_id) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s arg:%s\n", __FUNCTION__, $cat_id));
        $map_array = NAVT::load_map();

        // It's possible that we already added this once
        $have_item = NAVT::find_by_id(TYPE_CAT, $cat_id);
        if( !is_array($have_item) ) {
            // get the category
            $category = get_category($cat_id);
            if( is_object($category)) {
                // get the data from the form
                $new_item = NAVT::get_cat_formdata($category);
                if(is_array($new_item)) {
                    $map_array[] = $new_item;
                    NAVT::update_option(SERIALIZED_MAP, $map_array);
                    if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: category added: %s\n", __FUNCTION__, $cat_id));
                }
                else {
                    if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: call to get_cat_formdata did not produce new item\n", __FUNCTION__));
                }
            }//end if
            else {
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: category add failed: %s\n", __FUNCTION__, $cat_id));
            }
        }
        else {
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: cat id: %s already known - add ignored\n", __FUNCTION__, $cat_id));
        }

    }// end function

    /**
     * Delete a category from the menu map
     * This routine is registered with Word Press as an action (@see Word Press add_action('delete_category')
     *
     * @param integer $id Database identifier for the deleted category
	 */
    function delete_category_wpcb($cat_id) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s arg:%s\n", __FUNCTION__, $cat_id));

        $new_map = array();
        $was_removed = 0;
        $map_array = NAVT::load_map();

        foreach( $map_array as $itm ) {
            if( ($itm[TYP] == TYPE_CAT) && ($itm[IDN] == $cat_id) ) {
                $was_removed = 1;
            }
            else {
                // copy this item into the new map
                $new_map[] = $itm;
            }
        }// end for

        if( $was_removed ) {
            NAVT::update_option(SERIALIZED_MAP, $new_map);
        }
    }// end function

    /**
     * Category change
     * This routine is registered with Word Press as a filter (@see Word Press add_filter('cat_id_filter')
     *
     * @param number $cat_id
     * @param boolean $update = true (category updated)
     *
	 */
    function update_category_wpcb($cat_id, $update=false) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s args:%s, %d\n", __FUNCTION__, $cat_id, $update));
        $upd = 0;

        if( $update == true ) {
            // Updating existing category

            $updated_cat = array();

            // get the category
            $category = get_category($cat_id);
            if( is_object($category)) {

                // get the data from the update form
                $updated_cat = NAVT::get_cat_formdata($category);
                $map_array = NAVT::load_map();
                $upd = 0;

                for( $i = 0; $i < count($map_array); $i++ ) {

                    if( ($map_array[$i][TYP] == TYPE_CAT) && ($map_array[$i][IDN] == $cat_id) ) {
                        // copy any changes
                        $upd += (($map_array[$i][GRP] != $updated_cat[GRP]) ? 1: 0); $map_array[$i][GRP] = $updated_cat[GRP];
                        $upd += (($map_array[$i][TTL] != $updated_cat[TTL]) ? 1: 0); $map_array[$i][TTL] = $updated_cat[TTL];
                        $upd += (($map_array[$i][NME] != $updated_cat[NME]) ? 1: 0); $map_array[$i][NME] = $updated_cat[NME];
                        $upd += (($map_array[$i][OPT] != $updated_cat[OPT]) ? 1: 0); $map_array[$i][OPT] = $updated_cat[OPT];
                        $upd += (($map_array[$i][EXT] != $updated_cat[EXT]) ? 1: 0); $map_array[$i][EXT] = $updated_cat[EXT];
                        $upd += (($map_array[$i][LVL] != $updated_cat[LVL]) ? 1: 0); $map_array[$i][LVL] = $updated_cat[LVL];
                        $upd += (($map_array[$i][EX2] != $updated_cat[EX2]) ? 1: 0); $map_array[$i][EX2] = $updated_cat[EX2];
                        break;
                    }
                }
                // update using the same map
                NAVT::update_option(SERIALIZED_MAP, $map_array);
            }
            else {
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("category update failed: %s", $cat_id));
            }
        }

        return(upd ? $cat_id: 0);

    }// end function

    /**
     * Hook for category edit
     * This routine is registered with Word Press as an action (@see Word Press add_action('edited_category')
     *
     * @param integer $cat - category id
     * @since 95.38
     */
    function edited_category_wpcb($cat_id) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s cat id:%s \n", __FUNCTION__, $cat_id));

        $cat = get_category($cat_id);
        if( is_object($cat) ) {

            // We *should* know about this category
            $itm = NAVT::find_by_id(TYPE_CAT, $cat_id);

            if( !is_array($itm) || null == $itm ) {
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: - ignored(1): %s\n",__FUNCTION__, $cat_id));
            }
            else {
                // Update the category information
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("call to update the category\n"));
                NAVT::update_category_wpcb($cat_id, true);
            }
        }
        else {
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: - ignored(2): %s\n",__FUNCTION__, $cat_id));
        }
        return;
    }

    /**
     * Hook for category changes not related to category properties
     * This routine is registered with Word Press as an action (@see Word Press add_action('edit_category')
     *
     * @param integer $cat - category id
     * @since 95.37
     */
    function edit_category_wpcb($cat_id) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s cat id:%s \n", __FUNCTION__, $cat_id));

        $cat = get_category($cat_id);
        if( is_object($cat) ) {

            // If this is new then it was created on-the-fly from the post page
            $itm = NAVT::find_by_id(TYPE_CAT, $cat_id);

            if( !is_array($itm) || null == $itm ) {
                // we don't know about this category... since we can't provide
                // a form from the post we only have to fake one form field for the add routine
                // to add the category.
                $_POST['NAVT']['navgroup'] = NAVT::get_option(DEF_ADD_GROUP);
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: call to category add for cat: %s\n",__FUNCTION__, $cat_id));
                NAVT::add_category_wpcb($cat_id);
            }
            else {
                // We already know about this category
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: - ignored(1): %s\n",__FUNCTION__, $cat_id));
            }
        }
        else {
            // category was not found
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: - ignored(2): %s\n",__FUNCTION__, $cat_id));
        }
        return;
    }

    /**
     * Hook for adding my 2cents to the edit category page (Manage->categories)
     * This routine is registered with Word Press as an action (@see Word Press add_action('edit_category_form')
     *
     * @param object $category object
     * @since 95.30
     */
    function edit_category_form_wpcb($cat) {

        $itm = null;
        $submit_button = __('Save/Update &raquo;');
        if( !empty($cat) && !isBlank($cat->cat_ID) ) {
            $itm = NAVT::find_by_id(TYPE_CAT, $cat->cat_ID);
        }

        $grp = ((is_array($itm) && !empty($itm[GRP])) ? $itm[GRP]: ID_DEFAULT_GROUP);
        $nme = ((is_array($itm) && !empty($itm[NME])) ? $itm[NME]: '');
        $opt = intval( ((is_array($itm) && !empty($itm[OPT])) ? $itm[OPT]: '0') );
        $lvl = ((is_array($itm) && !empty($itm[LVL])) ? $itm[LVL]: '0');
        $ext = ((is_array($itm) && !empty($itm[EXT])) ? $itm[EXT]: '');
        $ex2 = ((is_array($itm) && !empty($itm[EX2])) ? $itm[EX2]: '');
        $ddlist = NAVT::build_group_dropdown($grp);
        $opt = intval($opt);

        // Added to the sidebar area of the page
        $anchor_type = ($opt & (STANDARD_LINK | TEXT_OVER_GRAPHIC | TEXT_WITH_SIDE_GRAPHIC | GRAPHIC_LINK));
?>
<!-- <?php echo SCRIPTNAME . ' v' . SCRIPTVERS; ?> -->
 <script type='text/javascript'>
 /* <![CDATA[ */
 addLoadEvent(navt_cat_options);
 function navt_cat_options() {
     jQuery.noConflict();
     init_navt_cat_options();
 };
 /* ]]> */
</script>
<div id="we_woke_up_next_to_a_table_ahhhhhhhhhh" class="wrap" style="position:relative;">
    <h2><?php _e('NAVT Category Options');?></h2>
       <div>
       <p><label> <?php _e('Navigation Group');?> <br /><?php echo $ddlist; ?></label></p>
       <p><label><?php _e('Anchor Display Option');?><br />
        <select id="navt_anchor_display_option_cat" name="NAVT[linkoptions]">
            <option value="<?php echo STANDARD_LINK;?>" <?php selected(STANDARD_LINK, $anchor_type);?>><?php _e('Standard Text Link');?></option>
            <option value="<?php echo TEXT_OVER_GRAPHIC;?>" <?php selected(TEXT_OVER_GRAPHIC, $anchor_type);?>><?php _e('Text over graphic');?></option>
            <option value="<?php echo TEXT_WITH_SIDE_GRAPHIC;?>" <?php selected(TEXT_WITH_SIDE_GRAPHIC, $anchor_type);?>><?php _e('Text with side graphic');?></option>
            <option value="<?php echo GRAPHIC_LINK;?>" <?php selected(GRAPHIC_LINK, $anchor_type);?>><?php _e('Graphic link');?></option>
        </select></label></p>
        <p><label id="navt_imgclass_lbl">Side graphic CSS Class<br />
        <input id="navt_imgclass_tbx" name="NAVT[imgclass]" type="text" size="20" maxlength="80" value="<?php echo $ext;?>" style="padding:3px;" /></label></p>
        <p><label>Alias Name<br />
        <input id="navt_item_alias" name="NAVT[catalias]" type="text" size="20" maxlength="50" value="<?php echo $nme;?>" style="padding:3px;"/></label><br /></p>
        <p><label>
        <select name="NAVT[level]" style="width:8em;">
            <option value="0" <?php selected('0', $lvl);?>><?php _e('Top');?></option>
            <option value="1" <?php selected('1', $lvl);?>>&mdash; 1</option>
            <option value="2" <?php selected('2', $lvl);?>>&mdash;&mdash; 2</option>
            <option value="3" <?php selected('3', $lvl);?>>&mdash;&mdash;&mdash; 3</option>
            <option value="4" <?php selected('4', $lvl);?>>&mdash;&mdash;&mdash;&mdash; 4</option>
        </select> Level</label></p>
        </div>

        <div style="position:absolute; top:60px; left:250px;">
        <p><label><input name="NAVT[showincat]" type="checkbox" value="1" <?php checked(SHOW_IN_LIST, ($opt & SHOW_IN_LIST));?> /> <?php _e('Also show in WP category list');?></label><br />
        <label id="navt_append_post_count_cb_lbl"><input name="NAVT[appendcount]" id="navt_append_post_count_cb" type="checkbox" value="1" <?php checked(APPEND_POST_COUNT, ($opt & APPEND_POST_COUNT));?> /> <?php _e('Append post count to link text');?></label><br />
        <label><input name="NAVT[showempty]" type="checkbox" value="1" <?php checked(SHOW_IF_EMPTY, ($opt & SHOW_IF_EMPTY));?> /> <?php _e('Show if empty');?></label><br />
        <label><input name="NAVT[usedesc]" type="checkbox" value="1" <?php checked(USE_CAT_DESC, ($opt & USE_CAT_DESC));?> /> <?php _e('Use category description as tooltip');?></label><br />
        <label><input name="NAVT[isprivate]" type="checkbox" value="1" <?php checked(ISPRIVATE, ($opt & ISPRIVATE));?> /> <?php _e('Private (User must be logged in to see)');?></label><br />
        <label><input name="NAVT[nofollow]"  type="checkbox" value="1" <?php checked(NOFOLLOW, ($opt & NOFOLLOW));?> /> <?php _e('rel=nofollow');?></label><br />
        <label><input name="NAVT[donotuse]"  type="checkbox" value="1" <?php checked(DO_NOT_DISPLAY, ($opt & DO_NOT_DISPLAY));?> /> <?php _e('Do not use');?></label></p>
        <p class="submit"><input type="submit" name="submit" value="<?php echo $submit_button;?>"/></p>
        </div>
</div>
<!--end-->
   <?php

    }// end function

    /**
     * Retrieve the data from the addcat or editcat wordpress form
     * @param  object $category
     * @return array - returns a formatted map item
     * @since 95.30
     */
    function get_cat_formdata($category) {

        // defaults...
        $add_item = $opt = 0;
        $itm = false;

        $ext = '';
        $ttl = $nme = str_replace('"', "'", $category->cat_name);
        $idn = $category->cat_ID;
        $grp = NAVT::get_option(DEF_ADD_GROUP);
        $typ = TYPE_CAT;
        $lvl = NAVT::get_category_level($idn);

        if( isset($_POST['NAVT'])) {
            foreach($_POST['NAVT'] as $option => $value) {
                switch( $option ) {
                    case 'navgroup':    {$grp = $value;                  break;}
                    case 'linkoptions': {$opt |= intval($value);         break;}
                    case 'imgclass':    {$ext = sanitize_title($value);  break;}
                    case 'catalias':    {$nme = (isBlank($value) ? $value: $nme); break;}
                    case 'level':       {$lvl = $value;                  break;}
                    case 'isprivate':   {$opt |= ISPRIVATE;              break;}
                    case 'donotuse':    {$opt |= DO_NOT_DISPLAY;         break;}
                    case 'showincat':   {$opt |= SHOW_IN_LIST;           break;}
                    case 'appendcount': {$opt |= APPEND_POST_COUNT;      break;}
                    case 'showempty':   {$opt |= SHOW_IF_EMPTY;          break;}
                    case 'usedesc':     {$opt |= USE_CAT_DESC;           break;}
                    case 'nofollow':    {$opt |= NOFOLLOW;               break;}
                }// end switch
            }// end foreach

            if( isBlank($nme) ) {
                $nme = $ttl;
            }

            if(!isBlank($nme)) {
                $nme = strip_tags($nme);
                $nme = stripcslashes($nme);
                $nme = str_replace('"', "'", $nme);
            }

            // Fix/correct link options
            if( ($opt & TEXT_WITH_SIDE_GRAPHIC)  && isBlank($ext) ) {
                $opt ^= TEXT_WITH_SIDE_GRAPHIC; // turn this off
            }

            // check that the group is legitimate
            $grp = NAVT::check_group($grp);
            $add_item = 1;
        }
        else {
            // No form data ... maybe handled by on-the-fly category add from post page
            // Add the category if we don't know about it

            $have_cat = NAVT::find_by_id(TYPE_CAT, $idn);
            if( !is_array($have_cat) ) {
                $grp = NAVT::check_group($grp);
                $add_item = 1;
            }
        }

        if( $add_item ) {
            $args = array(
            'grp' => $grp,
            'typ' => $typ,
            'idn' => $idn,
            'ttl' => $ttl,
            'nme' => $nme,
            'opt' => $opt,
            'ext' => $ext,
            'lvl' => $lvl,
            'ex2' => '');
            $itm = navt_format_menu_item($args);
        }
        return($itm);
    }

    /**
     * Hook for adding my 2cents to the edit user profile page (Manage->users)
     * This routine is registered with Word Press as an action (@see Word Press add_action('edit_user_profile')
     *
     * @since 95.38
     */
    function edit_user_profile_wpcb() {

        // get the item
        global $user_id;
        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s user id=%s\n", __FUNCTION__, $user_id));

        $itm = NAVT::find_by_id( TYPE_AUTHOR, $user_id);

        $grp = (is_array($itm) && !empty($itm[GRP]) ? $itm[GRP]: ID_DEFAULT_GROUP);
        $opt = intval( (is_array($itm) && !empty($itm[OPT]) ? $itm[OPT] : (SHOW_AVATAR | USE_DEF_AVATAR)) );
        $ext = (is_array($itm) && !empty($itm[EXT]) ? $itm[EXT] : '');
        $nme = (is_array($itm) && !empty($itm[NME]) ? $itm[NME] : $itm[TTL]);
        $lvl = (is_array($itm) && !empty($itm[LVL]) ? $itm[LVL] : 0);

        if( '' == $ext ) {
            $ext = NAVT::get_url() . '/' . IMG_AVATAR;
        }

        $avatar_list = NAVT::build_avatar_list(1, 0,'', $ext, 0);
        $ddlist = NAVT::build_group_dropdown($grp);

?>
<!-- <?php echo SCRIPTNAME . ' v' . SCRIPTVERS; ?> -->

<style type="text/css">
fieldset#navt_user_options img {padding: 5px;margin-bottom: 1em;float:left;overflow:hidden;border: 1px solid #777;width: 32px;height: 32px;}
fieldset#navt_user_options input[type="checkbox"] {padding: 0 !important;width: 32px !important;}
fieldset#navt_user_options input[type="text"] {padding: 0; font-size: 1em; padding: 3px; width:95%;}
fieldset#navt_user_options input#navt_user_alias {width: 40%;}
#navt_user_options label {padding:0;margin:0;}
#navt_user_options p {line-height:normal;padding:0;}
</style>
 <script type='text/javascript'>
 /* <![CDATA[ */
 addLoadEvent(navt_user_options);
 function navt_user_options() {
     jQuery.noConflict();
     init_navt_usr_options();
 }
 /* ]]> */
</script>
    <fieldset id="navt_user_options">
        <legend><?php _e('NAVT User Info');?></legend>
        <p><label> <?php _e('Navigation Group');?> <br /><?php echo $ddlist; ?></label></p>
        <p><label><?php _e('Menu Alias');?><br /><input id="navt_item_alias" type="text" name="NAVT[useralias]" maxlength="80" value="<?php echo $nme;?>" /></label></p>
        <p><label><?php _e('Avatar');?> <img id="navt_avatar_image" src="<?php echo $ext; ?>"></label></p>
        <p><label><input type="checkbox" value="1" id="navt_show_avatar_cb" name="NAVT[showavatar]" <?php checked(SHOW_AVATAR, $opt & SHOW_AVATAR); ?> /> <?php _e('Show Avatar');?></label><br />
        <label id="navt_use_default_avatar_cb_lbl"><input type="checkbox" value="1" name="NAVT[usedefavatar]" id="navt_use_default_avatar_cb" <?php checked(USE_DEF_AVATAR, $opt & USE_DEF_AVATAR);?> /> <?php _e('Use default');?> </label></p>
        <p><label id="navt_default_avatar_select_lbl"><?php _e('Default Avatar Selection');?><br />
           <select id="navt_default_avatar_select" name="NAVT[avatarlist]">
              <?php echo $avatar_list; ?>;
           </select></label></p>
        <p><label id="navt_user_avatar_txb_lbl"><?php _e('Avatar URL');?><br /><input type="text" id="navt_user_avatar_txb" name="NAVT[avatar_url]" maxlength="80" value="<?php echo $ext;?>" /></label></p>
        <p><label>
        <select name="NAVT[level]" style="width:8em;">
            <option value="0" <?php selected('0', $lvl);?>><?php _e('Top');?></option>
            <option value="1" <?php selected('1', $lvl);?>>&mdash; 1</option>
            <option value="2" <?php selected('2', $lvl);?>>&mdash;&mdash; 2</option>
            <option value="3" <?php selected('3', $lvl);?>>&mdash;&mdash;&mdash; 3</option>
            <option value="4" <?php selected('4', $lvl);?>>&mdash;&mdash;&mdash;&mdash; 4</option>
        </select> Level</label></p>
        <p><label id="navt_append_post_count_cb_lbl"><input type="checkbox" id="navt_append_post_count_cb" name="NAVT[appendpostcount]" value="1" <?php checked(APPEND_POST_COUNT, $opt & APPEND_POST_COUNT);?>/> <?php _e('Append post count to link');?></label><br />
        <label><input type="checkbox" name="NAVT[showifempty]" value="1" <?php checked(SHOW_IF_EMPTY, $opt & SHOW_IF_EMPTY);?>/> <?php _e('Show if no posts');?></label><br />
        <label id="navt_inc_email_address_cb_lbl"><input type="checkbox" name="NAVT[incemail]" id="navt_inc_email_address_cb" value="1" <?php checked(INC_EMAIL, $opt & INC_EMAIL);?>/> <?php _e('Include email address');?></label><br />
        <label id="navt_inc_website_address_cb_lbl"><input type="checkbox" name="NAVT[incwebsite]" id="navt_inc_website_address_cb" value="1" <?php checked(INC_WEBSITE, $opt & INC_WEBSITE);?>/> <?php _e('Include website address');?></label><br />
        <label id="navt_inc_bio_cb_lbl"><input type="checkbox" name="NAVT[incbio]" id="navt_inc_bio_cb" value="1" <?php checked(INC_BIO, $opt & INC_BIO);?>/> <?php _e('Include bio summary');?></label><br />
        <label id="navt_hide_linktext_cb_lbl"><input type="checkbox" name="NAVT[hidelinktext]" id="navt_hide_linktext_cb" value="1" <?php checked(NO_LINK_TEXT, $opt & NO_LINK_TEXT);?>/> <?php _e('Hide link text');?></label><br />
        <label><input name="NAVT[nofollow]"  type="checkbox" value="1" <?php checked(NOFOLLOW, ($opt & NOFOLLOW));?> /> <?php _e('rel=nofollow');?></label><br />
        <label><input name="NAVT[donotuse]"  type="checkbox" value="1" <?php checked(DO_NOT_DISPLAY, ($opt & DO_NOT_DISPLAY));?> /> <?php _e('Do not use');?></label></p>
        <input type="hidden" id="navt_users_saved_avatar_url" value="<?php echo $ext;?>" />

    </fieldset>

<!--end-->
<?php
// NOTE: On 'submit' we end up in profile_update_wpcb
    }

    /**
     * Hook for updates to the user profile
     * This routine is registered with Word Press as an action (@see Word Press add_action('profile_update')
     *
     * @since 95.38
     * @param integer $user_id - the user id
     */
    function profile_update_wpcb($user_id) {

        $uid = ( isset($_REQUEST['user_id']) ) ? $_REQUEST['user_id'] : $user_id;

        if( !empty($uid) ) {
            $itm = NAVT::find_by_id( TYPE_AUTHOR, $uid);
            if(is_array($itm)) {

                $userdata = get_userdata($itm[IDN]);
                if( !isBlank($userdata->first_name) || !isBlank($userdata->last_name) ) {
                    $t = str_replace('"', "'", ($userdata->first_name . ' ' . $userdata->last_name));
                    $n_nme = ($t != $itm[NME]) ? $t : $itm[NME];
                }
                $alias_change = 0;

                if( isset($_REQUEST['NAVT']) ) {
                    $map_array = NAVT::load_map();
                    $idx = NAVT::get_item_index(TYPE_AUTHOR, $uid, $map_array);
                    $new_item = array();

                    $n_opt = 0;
                    $n_default_avatar = '';
                    $n_lvl = 0;
                    $n_ext = '';
                    $ttl = $itm[TTL]; // this can't change (user login id)
                    $idn = $itm[IDN]; // this can't be changed either

                    foreach($_REQUEST['NAVT'] as $option => $value) {
                        switch($option) {
                            case 'useralias': {
                                if( !isBlank($value) ) {
                                    $t = stripslashes($value);
                                    $n_nme = htmlspecialchars($t);
                                    $alias_change = 1;
                                }
                                break;
                            }

                            case 'showavatar':       $n_opt |= SHOW_AVATAR;  break;
                            case 'usedefavatar':     $n_opt |= USE_DEF_AVATAR;  break;
                            case 'avatar_url':       $t = strip_tags($value); $n_ext = stripslashes($t); break;
                            case 'appendpostcount':  $n_opt |= APPEND_POST_COUNT;  break;
                            case 'hidelinktext':     $n_opt |= NO_LINK_TEXT;  break;
                            case 'navgroup':         $n_grp = $value; break;
                            case 'showifempty':      $n_opt |= SHOW_IF_EMPTY; break;
                            case 'avatarlist':       $n_default_avatar = $value; break;
                            case 'donotuse':         $n_opt |= DO_NOT_DISPLAY;  break;
                            case 'level':            $n_lvl = $value; break;
                            case 'incemail':         $n_opt |= INC_EMAIL; break;
                            case 'incwebsite':       $n_opt |= INC_WEBSITE; break;
                            case 'incbio':           $n_opt |= INC_BIO; break;
                            case 'nofollow':         $n_opt |= NOFOLLOW; break;
                        }
                    } // end foreach

                    // adjustments for the avatar
                    if( isBlank($n_ext) && ($n_opt & SHOW_AVATAR) ) {
                        if( $n_opt & USE_DEF_AVATAR ) {
                            $n_ext = $n_default_avatar;
                        }
                        else {
                            // no avatar and not using default... turn off avatar usage altogether
                            $n_opt ^= SHOW_AVATAR; // turn this off
                            if( $n_opt & NO_LINK_TEXT ) {
                                $n_opt ^= NO_LINK_TEXT; // turn this off
                            }
                        }
                    }

                    // this can't be blank or empty.
                    if( isBlank($n_nme) ) {
                        $n_nme = str_replace('"', "'", $itm[TTL]);
                    }

                    // create a new navigation item from the information and add it to the map
                    $args = array(
                    'grp' => $n_grp,
                    'typ' => TYPE_AUTHOR,
                    'idn' => $idn,
                    'ttl' => $ttl,
                    'nme' => $n_nme,
                    'opt' => $n_opt,
                    'ext' => $n_ext,
                    'lvl' => $n_lvl,
                    'ex2' => '');
                    $new_item = navt_format_menu_item($args);
                    $map_array[$idx] = $new_item;
                    NAVT::update_option(SERIALIZED_MAP, $map_array);
                } // is post
            }// end is_array($itm)
        }// is not empty
    }// end function

    /**
     * Hook for adding new users
     *
     * This routine is registered with Word Press as an action (@see Word Press add_action('profile_update')
     * @param integer $user_id - new user id
     * @since 95.38
     */
    function user_register_wpcb($user_id) {

        global $wpdb;
        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s user id=%s\n", __FUNCTION__, $user_id));
        $map_array = NAVT::load_map();
        $itm = NAVT::find_by_id( TYPE_AUTHOR, $user_id);
        $opt = (SHOW_AVATAR | USE_DEF_AVATAR);

        if(!is_array($itm)) {
            // new user
            $args = array();
            $userdata = get_userdata($user_id);
            $level = intval(get_usermeta($user_id, $wpdb->prefix . 'user_level'));
            $opt |= ($level > 1) ? 0: DO_NOT_DISPLAY;

            $nme = ('' != $userdata->first_name && '' != $userdata->last_name) ?
            $userdata->first_name . ' ' . $userdata->last_name : $userdata->user_nicename;
            $nme = ( '' == $nme ) ? str_replace('"', "'", $userdata->user_login): str_replace('"', "'", $nme);

            $args['grp'] = ID_DEFAULT_GROUP;
            $args['typ'] = TYPE_AUTHOR;
            $args['idn'] = $user_id;
            $args['ttl'] = $userdata->user_login;
            $args['nme'] = $nme;
            $args['opt'] = $opt;
            $args['lvl'] = '0';
            $args['ex2'] = '';
            $args['ext'] = NAVT::get_url() . '/' . IMG_AVATAR;

            $map_array[] = navt_format_menu_item($args);
            NAVT::update_option(SERIALIZED_MAP, $map_array);
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s user '%s' added\n", __FUNCTION__, $args['ttl']));
        }
    }
    /**
     * Hook for removing users
     *
     * This routine is registered with Word Press as an action (@see Word Press add_action('delete_user')
     * @param integer $user_id - user id
     * @since 95.38
     */
    function delete_user_wpcb($user_id) {

        $new_map = array();
        $was_removed = 0;
        $map_array = NAVT::load_map();

        foreach( $map_array as $itm ) {
            if( ($itm[TYP] == TYPE_AUTHOR) && ($itm[IDN] == $user_id) ) {
                $was_removed = 1;
            }
            else {
                // copy this item into the new map
                $new_map[] = $itm;
            }
        }// end for

        if( $was_removed ) {
            NAVT::update_option(SERIALIZED_MAP, $new_map);
        }
    }

    /**
     * Append excluded categories to the database exclusion list
     * This routine is registered with Word Press as a filter (@see Word Press add_filter('list_cats_exclusions')
     *
     * This routine appends the necessary database 'AND' statements to the provided $exclusions string to prevent Word Press
     * from displaying category links for categories (and their children) that are part of a menu. For example, if the
     * category named 'Photos' were added to web site menu that appears on every page then the 'Photos' category would be
     * excluded from the list of categories returned by the Word Press function: 'wp_categories'. @See the plugin documentation
     * to find out how you display children categories of 'menu configured' parent categories.
     *
     * @param string $exclusions (category AND statements from other plugins)
     * @return string $exclusions (AND statements appended to the existing string)
	 */
    function list_cats_exclusions_wpcb($exclusions) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, (sprintf("%s arg:%s \n", __FUNCTION__, $exclusions)));
        $map_array = NAVT::load_map();
        $cat_excl = NAVT::get_exclusions($map_array);

        if( !is_admin() ) {
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("\t\tprocessing..\n"), $cat_excl);

            if( !empty($cat_excl) || !is_array($cat_excl) ) {
                $cats  = get_all_category_ids();
                for( $i = 0; $i < count($cats); $i++ ) {
                    if( in_array($cats[$i], $cat_excl) ) {
                        $exclusions .= ' AND cat_ID <> ' . $cats[$i];
                    }
                }// end for
            }// end if
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, (sprintf("\t\treturning: %s\n", $exclusions)));
        }// end if
        else {
            if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("\tinside admin flag .. ignored\n"));
        }
        return $exclusions;
    }// end function

    /**
     * Returns the value of the option identified by 'key'
     * Calls the Word Press function by the same name
     *
     * @param string_type $key
     * @return mixed value of key
     */
    function get_option($key) {
        return(get_option($key));
    }

    /**
     * Adds the key/Value and description to the database
     * Calls the Word Press function by the same name
     *
     * @param string $key
     * @param mixed $value
     * @param string $description of option
     */
    function add_option($key, $value, $description=NULL) {
        add_option($key, $value, $description);
    }

    /**
     * Updates the value of 'key' in the database
     * Calls the Word Press function by the same name
     *
     * @param string $key
     * @param mixed $value of key
     */
    function update_option($key, $value) {
        global $navt_map;

        if( $key == SERIALIZED_MAP ) {
            $navt_map = array(); // this will force a new get_option call for the ma
        }
        update_option($key, $value);
    }

    /**
     * Deletes the option identified by 'key' from the database
     * Calls the Word Press function by the same name
     *
     * @param string $key
     */
    function delete_option($key) {
        delete_option($key);
    }

    function check_group($name) {
        $map = NAVT::load_map();
        $gnames = NAVT::get_groups($map);
        $ret_name = ( !in_array($name, $gnames) ) ? ID_DEFAULT_GROUP: $name;
        return($ret_name);
    }

    /**
     * Find a menu item in the map array by type and id
     *
     * @param string $type - TYPE_CAT, TYPE_PAGE, TYPE_AUTHOR
     * @return array - returns the item entry from the map (if found) or null
     * @since 95.30
     */
    function find_by_id($type, $idn) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s:  type: %s, idn: %s\n", __FUNCTION__, $type, $idn));
        $ar = NAVT::load_map();
        if( is_array($ar) && count($ar) > 0 ) {
            for( $i = 0; $i < count($ar); $i++ ) {
                if( $type == $ar[$i][TYP] && $idn == $ar[$i][IDN] ) {
                    if( isBlank($ar[$i][GRP]) || isBlank($ar[$i][TTL]) ) {
                        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: BAD ITEM!", __FUNCTION__), $itm);
                        continue;
                    }
                    else {
                        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: returning", __FUNCTION__), $ar[$i]);
                        return($ar[$i]);
                    }
                }
            }
        }
        else {
            navt_write_debug(NAVT_WPHOOKS, sprintf("%s: map is empty or not an array\n", __FUNCTION__));
        }
        navt_write_debug(NAVT_WPHOOKS, sprintf("%s: not found\n", __FUNCTION__));
        return(null);
    }

    /**
     * Find a menu item a map array by type and id and return the index
     *
     * @param string $type - TYPE_CAT, TYPE_PAGE
     * @param string $name - title of the item
     * @param array $map - navigation map
     * @return array - returns the map array index
     * @since 95.30
     */
    function get_item_index($type, $idn, $map) {

        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf(">> %s: locate type: %s, idn: %s in map\n", __FUNCTION__, $type, $idn));
        $ret = -1;

        for($i = 0; $i < count($map); $i++ ) {
            if( ($map[$i][TYP] == $type) && ($map[$i][IDN] == $idn) ) {
                $ret = $i;
                break;
            }
        }
        if(NAVTdbg()) {
            if( -1 != $ret ) {
                navt_write_debug(NAVT_WPHOOKS, sprintf("%s: << returning:\n", __FUNCTION__), $map[$ret]);
            }
            else {
                if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("%s: << returning: %s\n", __FUNCTION__, $ret));
            }
        }
        return($ret);
    }

    /**
     * Build a HTML dropdown list containing the navigation groups
     *
     * @param string $curgrp -  selected group
     * @return string - HTML <select> string
     * @since 95.30
     */
    function build_group_dropdown($curgrp) {
        $ddlist = sprintf("\t<select name='NAVT[navgroup]'>\n");
        $map_array = NAVT::load_map();
        $grps = NAVT::get_groups($map_array);

        for( $i = 0; $i < count($grps); $i++ ) {
            $select = ($curgrp == $grps[$i] ? ' selected="selected"':'');
            $ddlist .= sprintf("\t   <option value=\"%s\"%s>%s</option>\n", $grps[$i], $select, strtolower($grps[$i]));
        }
        $ddlist .= sprintf("\t</select>\n\t");
        return($ddlist);
    }

    /**
     * Determines if the navigation group we're about to build
     * has no styles
     *
     * @param string $name - name of group
     * @param array $map - the map array
     * @returns 1 if the group has no style, 0 (zero) otherwise
     */
    function is_nostyle($name, $map) {

        $retcode = 0;
        // Locate the first navigation item in the group - each navigation item
        // contains the nostyle bit... we just need one
        foreach( $map as $itm ) {
            $opt = intval($itm[OPT]);
            if( ($name == $itm[GRP]) && ($opt & HAS_NOSTYLE) ) {
                $retcode = 1;
                break;
            }
        }
        return($retcode);
    }

    /**
     * Truncates text to $n length and adds an elipse
     *
     * @param string $text
     * @param integer $numb
     */
    function truncate($text, $n) {
        // source: www.kigoobe.com
        if (strlen($text) > $n) {
            $text = substr($text, 0, ($n-3));
            //$text = substr($text,0, strrpos($text," "));
            $text = $text.'...';
        }
        return $text;
    }

    /** ----------------------------------------
     * Builds requested navigation lists (API)
     * -----------------------------------------
     *
     * @param string $sGroupName
     * @param string $sTitle
     * @param string $sBeforeGroup
     * @param string $sAfterGroup
     * @param string $sBeforeItem
     * @param string $sAfterItem
     */
    function getlist($sNavGroupName, $sTitle, $sBeforeGroup, $sAfterGroup, $sBeforeItem, $sAfterItem) {

        $in = 2;
        $ng = $tmp = array();
        $out = $before_text = $after_text = '';
        $seq = 0;
        $sAfterItem = "<$sAfterItem>";

        if( !empty($sNavGroupName) && '' != $sNavGroupName ) {

            // TODO... add the ability for user to indicate which pages/archives/posts/etc this
            // navigation group can be displayed on. This concept supported by SBM but not by
            // wordpress widgets. SBM 'standalone' is an abandoned project.

            $all_nav_items = NAVT::load_map();
            $is_nostyle = NAVT::is_nostyle($sNavGroupName, $all_nav_items);

            if(empty($sBeforeGroup) || isBlank($sBeforeGroup)) {
                $sAfterGroup = '';
            }
            else {
                $sGroupName = strtolower($sNavGroupName);
                $sB = $sBeforeGroup;
                $sA = $sAfterGroup;
                $sBeforeGroup = sprintf("<%s %s>", $sBeforeGroup, set_klass($is_nostyle, "class='menu $sGroupName'"));
                $sAfterGroup  = sprintf("%s<%s><!--/%s-->\n\n", _indentt($in), $sAfterGroup, SCRIPTNAME);
            }

            $before_text = sprintf("\n\n%s<!-- %s v%s -->\n", _indentt($in), SCRIPTNAME, SCRIPTVERS);
            if( !isBlank($sTitle) ) $before_text .= sprintf("%s%s\n", _indentt($in), $sTitle);
            if( !isBlank($sBeforeGroup) ) $before_text .= sprintf("%s%s\n", _indentt($in), $sBeforeGroup);
            $after_text = $sAfterGroup;

            // collect all of the items in the named group
            $is_select = 0;
            $sNavGroupName = strtolower($sNavGroupName); // force this to lowercase
            foreach( $all_nav_items as $nav_item ) {

                // case insensitive name compare
                if( strcasecmp($nav_item[GRP], $sNavGroupName) || !strcasecmp($sNavGroupName, ID_DEFAULT_GROUP) ) {
                    continue;
                }

                $nav_item[GRP] = strtolower($nav_item[GRP]); // force group names to lowercase
                $opt = intval($nav_item[OPT]);
                if($opt & HAS_DD_OPTION) { $is_select = 1; }
                $ng[] = $nav_item;
            }

            // preprocess the group contents
            $tmp = (($is_select) ? NAVT::calc_selects($ng) : NAVT::calc_nest($ng));

            // Start creating the list
            $in++;
            $form_open = $form_id = $nest_level = 0;
            $forms = $tags = array();
            $listout = '';
            foreach( $tmp as $idx => $item ) {

                if( !$is_select ) {

                    if( isset($item['tok']) ) {

                        if( $item['tok'] == 's-ul' ) {
                            $in++; $name = $item['nm']; $sub =  $item['sb'];
                            $listout .= sprintf("%s<%s %s>\n", _indentt($in), $sB,
                            set_klass($is_nostyle, "class='$name $sNavGroupName-sublevel-$sub'"));
                            $in++;
                        }
                        elseif( $item['tok'] == 'e-ul' ) {
                            $in--;  $listout .= sprintf("%s<%s>\n", _indentt($in), $sA); $in--;
                        }
                        elseif( $item['tok'] == 's-li' ) {
                            $nav_item = $item['item'];
                            $listout .= NAVT::mk_navlist($nav_item, $sBeforeItem, '', $in, $seq++, $is_nostyle);
                        }
                        elseif( $item['tok'] == 'e-li' ) {
                            $listout .=  sprintf("%s%s\n", _indentt($in), $sAfterItem);
                        }
                        elseif( $item['tok'] == 's-li-e-li' ) {
                            $nav_item = $item['item'];
                            $listout .= NAVT::mk_navlist($nav_item, $sBeforeItem, $sAfterItem, $in, $seq++, $is_nostyle);
                        }
                        else {
                            if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s ??? token: %s\n", __FUNCTION__, $item['tok']));
                        }
                    }
                    else {
                        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s unset token\n", __FUNCTION__), $item);
                    }
                }
                else {

                    // Select format processing
                    if( isset($item['tok']) ) {
                        if( $item['tok'] == 's' ) {
                            // Start a form with a select
                            $forms[$form_id] .= NAVT::mk_form($sNavGroupName, $form_id, $in+2, $is_nostyle);
                            $form_open = 1; // form is open
                        }
                        elseif( $item['tok'] == 'e' ) {
                            $forms[$form_id] .= NAVT::end_form($in+2);
                            $form_open = 0; // form is closed
                            $form_id++; // advance the form number
                        }
                        else {
                            // just one of the items
                            $nav_item = $item['tok'];
                            if( is_array($nav_item) ) {
                                $forms[$form_id] .= NAVT::mk_ddoption($nav_item, $in+4, $is_nostyle);
                            }
                        }
                    }
                }
            }// end for

            //
            // Final stage
            //

            // select item(s)
            if( $is_select ) {
                if( $form_open ) {
                    $forms[$form_id] .= NAVT::end_form($in+1);
                }

                // if the select is to be wrapped in a 'ul' or 'ol' then it must be inside an 'li'
                $is_ul = strpos($before_text, '<ul');
                $is_ol = strpos($before_text, '<ol');
                if( $is_ul !== false || $is_ol !== false ) {
                    // wrap it in a list item
                    $before_text .= _indentt($in+1) . '<li>' . "\n";
                    $after_text  = _indentt($in+1) . '</li>' . "\n" . $after_text;
                }

                $out = $before_text;
                for( $i = 0; $i < count($forms); $i++ ) {
                    $out .= $forms[$i];
                }
                $out .= $after_text;
                if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s output: %s\n", __FUNCTION__, $out));
            }
            else {
                // simple list
                $out = $before_text . $listout . $after_text;
                if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s output: %s\n", __FUNCTION__, $out));
            }
        } // end if
        else {
            // bad group name comment
            $out = _indentt($in);
            $out .= sprintf("<!-- NAVT:empty or unknown group name: '%s' -->\n", strtolower($sNavGroupName));
        }

        return($out);

    }// end navt_getlist

    /**
     * Group preprocessing
     * Builds a set of directives for creating the list
     * @param array $ng - contents of the group
     * @return array
     */
    function calc_nest($ng) {

        $tmp = array();
        $i = $open_ul = $open_li = $complete = $nlevel = 0;

        for( $idx = 0; $idx < count($ng); $idx++ ) {
            $c = intval($ng[$idx][LVL]);
            $n = ($idx+1 >= count($ng) ? 0: intval($ng[$idx+1][LVL]));

            if( $n > $c ) {

                // next item will be nested
                // start ths current item (but don't terminate the tag)
                $tmp[$i]['tok'] = 's-li';
                $tmp[$i++]['item'] = $ng[$idx];
                $open_li++;

                // start a new level
                $tmp[$i]['tok'] = 's-ul';
                $tmp[$i]['nm'] = sanitize_title($ng[$idx][TTL]);
                $tmp[$i++]['sb'] = ++$nlevel;
                $open_ul++;
            }

            elseif( $n < $c ) {
                // terminate this nested list
                // first add the current item
                $tmp[$i]['tok'] = 's-li-e-li';
                $tmp[$i++]['item'] = $ng[$idx];

                // rewind to next item level
                while( $nlevel > $n ) {

                    if( $open_ul > 0 ) {
                        $tmp[$i++]['tok'] = 'e-ul';
                        $open_ul--;
                    }
                    if($open_li > 0) {
                        $tmp[$i++]['tok'] = 'e-li';
                        $open_li--;
                    }
                    $nlevel--;
                }
            }

            else {
                // no change in level
                // just add the item
                $tmp[$i]['tok'] = 's-li-e-li';
                $tmp[$i++]['item'] = $ng[$idx];
            }
        }

        // Unwind the nests
        $complete = 0;
        while( !$complete ) {

            if( $open_ul > 0 ) {
                $tmp[$i++]['tok'] = 'e-ul';
                $open_ul--;
            }
            if($open_li > 0) {
                $tmp[$i++]['tok'] = 'e-li';
                $open_li--;
            }

            $complete = ($open_ul == 0) ? 1: 0;
        }

        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s\n", __FUNCTION__), $tmp);
        return($tmp);
    }

    /**
     * Group preprocessing
     * Builds a set of directives for creating a select dropdown
     *
     * @param array $ng - contents of a group
     * @return array
     */
    function calc_selects($ng) {

        // for select menus - calculate selects
        $tmp = array();
        $nidx = $sm = 0;
        foreach( $ng as $idx => $item ) {
            if( !$sm ) {
                if( $item[TYP] == TYPE_SEP ) {
                    // just add the item - outside of a select
                    $tmp[$nidx++]['tok'] = $item;
                }
                else {
                    // add the item before the select begins
                    $tmp[$nidx++]['tok'] = 's';
                    $tmp[$nidx++]['tok'] = $item;
                    $sm = 1;
                }
                continue;
            }
            else {
                if( $item[TYP] == TYPE_SEP ) {
                    // end the select then add the item
                    $tmp[$nidx++]['tok'] = 'e';
                    $tmp[$nidx++]['tok'] = $item;
                    $sm = 0;
                    continue;
                }
            }
            // add the item
            $tmp[$nidx++]['tok'] = $item;
        }

        if( $sm ) {
            $tmp[$nidx++]['tok'] = 'e';
        }

        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s\n", __FUNCTION__), $tmp);
        return($tmp);
    }

    /**
     * Creates a non-dropdown version of a navigation item (API)
     *
     * @param array $nav_item
     * @param string $sBeforeItem
     * @param string $sAfterItem
     * @param integer $in
     * @param integer $seq
     * @return returns a HTML formatted string
     */
    function mk_navlist($nav_item, $sBeforeItem, $sAfterItem, $in, $seq, $is_nostyle) {

        $li = '';
        global $wp_query;
        $qo = $wp_query->get_queried_object();

        // @since 95.38
        if( !empty($qo->cat_ID) ) {
            $loc = TYPE_CAT . $qo->cat_ID;
        }
        elseif( !empty($qo->ID) ) {
            if( !empty($qo->post_type) && ($qo->post_type == 'page') ) {
                $loc = TYPE_PAGE . $qo->ID;
            }
            elseif(!empty($qo->user_login) && is_author() ) {
                $loc = TYPE_AUTHOR . $qo->ID;
            }
        }
        else {
            if( is_home() ) {
                $loc = TYPE_LINK . HOMEIDN;
            }
        }

        $can_show = 0;
        $idn = $nav_item[IDN];
        $opt = intval($nav_item[OPT]);
        $typ = $nav_item[TYP];

        $is_private = ($opt & ISPRIVATE) ? 1: 0;
        if( $is_private ) { $can_show = is_user_logged_in() ? 1: 0; } else $can_show = 1;
        $can_show = ($opt & DO_NOT_DISPLAY) ? 0: $can_show;

        // short this out
        if( (!$can_show) || (($typ == TYPE_PAGE) && ($opt & ISDRAFTPAGE)) ) {
            ;// nothing to do here
        }
        else {
            // @since 95.31
            $active = (($loc == $typ.$idn) ? 1: 0);
            $maybe_blog_page = get_option(ALTBLOGPAGE);
            if( is_home() && !empty($maybe_blog_page) && $idn == $maybe_blog_page) {
                $active = 1;
            }

            // @since 95.31
            if($idn == HOMEIDN) {
                if((get_option('show_on_front')) == 'page' ) {
                    $page_on_front = get_option('page_on_front');
                    $active = ( $qo->ID == $page_on_front );
                }
            }

            // apply all of the item classes
            $item_class = (($seq % 2 > 0) ? 'orow' : 'erow') . ' ' . $nav_item[GRP] . '_item' . ' ' .
            /* (CLS_LEVEL . $nav_item[LVL] ) . ' ' . */ ($active ? CURPAGEITEM : PAGEITEM) . ($is_private ? ' private' : '' );

            // Common parameters for creating items
            $args = array(
            'nav_item'=>$nav_item,
            'item_class' => $item_class,
            'sBeforeItem' => $sBeforeItem,
            'sAfterItem' => $sAfterItem,
            'in' => $in,
            'is_nostyle' => $is_nostyle,
            'img_src' => NAVT::get_url() . '/' . IMG_BLANK,
            'tags' => (($opt & NOFOLLOW) ? " rel='nofollow'" : ''),
            'post_count' => '',
            'tooltip' => $nav_item[NME],
            'alt_text' => $nav_item[NME],
            'usr_defined' => $nav_item[EXT]
            );

            switch( $typ ) {
                case TYPE_CAT:    { $li = NAVT::mk_category_item($args); break; }
                case TYPE_PAGE:   { $li = NAVT::mk_page_item($args); break; }
                case TYPE_LINK:   { $li = NAVT::mk_link_item($args); break; }
                case TYPE_SEP:    { $li = NAVT::mk_divider_item($args); break; }
                case TYPE_ELINK:  { $li = NAVT::mk_uri_item($args);  break; }
                case TYPE_AUTHOR: { $li = NAVT::mk_author_item($args);  break; }
            }// end switch item type
        }

        return($li);
    }

    /**
     * Create a category item for the navigation group
     *
     * @param array   $args
     * @return  HTML string
     */
    function mk_category_item($args) {

        extract($args);

        $li = '';
        $opt = intval($nav_item[OPT]);
        $idn = intval($nav_item[IDN]);
        $cat = get_category($idn);
        $num_posts = $cat->category_count;

        $args['anchor_class'] = 'navt_clink';
        $args['link_url'] = get_category_link($idn);
        $args['item_class'] = ($item_class .= ' ' . $cat->category_nicename . '-cat ' . TAB_CATEGORY);
        $args['img_class'] = $cat->category_nicename . '-img';

        if( $opt & USE_CAT_DESC ) {
            $args['tooltip'] = $cat->category_description;
        }

        if( $opt & SHOW_IF_EMPTY || $num_posts > 0 ) {

            if( $opt & GRAPHIC_LINK ) {
                $args['anchor_class'] = 'navt_glink';
                $li = NAVT::mk_graphic_link($args);
            }
            else {
                // get post count and tooltip
                $args['post_count'] = (($opt & APPEND_POST_COUNT) ?
                sprintf("<span %s>(%s)</span>", set_klass($is_nostyle, "class='catcount'"), $num_posts) : '');

                // Link text with a side graphic
                if( $opt & TEXT_WITH_SIDE_GRAPHIC ) {
                    $li = NAVT::mk_text_with_side_graphic($args);
                }
                // link with text over graphic
                else if( $opt & TEXT_OVER_GRAPHIC ) {
                    $li = NAVT::mk_text_over_graphic($args);
                }
                else {

                    /*
                    * Output: (with styles)
                    *
                    *  <li class="$item_class"><a class="navt_clink" href="$cat_link" title="$nme">$nme</a></li>
                    */
                    extract($args); // get these again
                    $li = sprintf("%s<%s %s><a %s href='%s' title='%s' %s>%s%s</a>%s\n",
                    _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"),
                    set_klass($is_nostyle, "class='$anchor_class'"), $link_url,
                    $tooltip, $tags, $alt_text, $post_count, $sAfterItem);
                }
            }
        }

        return( $li );
    }

    /**
     * Create a page item for the navigation group
     *
     * @param array   $args
     * @return  HTML string
     */
    function mk_page_item($args) {

        extract($args);

        $li = '';
        $opt = intval($nav_item[OPT]);
        $idn = intval($nav_item[IDN]);

        $page = &get_post($idn);
        $args['item_class'] .= ' ' . $page->post_name . '-page ' . TAB_PAGE;
        $args['img_class'] = $page->post_name . '-img ';
        $args['link_url'] = get_permalink($idn);
        $args['anchor_class'] = 'navt_plink';

        if( $opt & GRAPHIC_LINK ) {
            $args['anchor_class'] = 'navt_glink';
            $li = NAVT::mk_graphic_link($args);
        }
        else {
            // Link text with a side graphic
            if( $opt & TEXT_WITH_SIDE_GRAPHIC ) {
                $li = NAVT::mk_text_with_side_graphic($args);
            }
            // link with text over graphic
            else if( $opt & TEXT_OVER_GRAPHIC ) {
                $li = NAVT::mk_text_over_graphic($args);
            }
            else {

                /* Plain Text
                * Output:
                *
                *  <li class="$item_class"><a class="navt_plink" href="permalink" title="$nme">$nme</a></li>
                */
                extract($args);  // get these again
                $li = sprintf("%s<%s %s><a %s href='%s' title='%s '%s>%s</a>%s\n",
                _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"),
                set_klass($is_nostyle, "class='$anchor_class'"), $link_url, $alt_text, $tags, $alt_text, $sAfterItem);
            }
        }
        return( $li );
    }

    /**
     * Create a author item for the navigation group
     *
     * @param array   $nav_item
     * @param string  $item_class
     * @param string  $sBeforeItem
     * @param string  $sAfterItem
     * @param integer $in
     * @return  HTML string
     */
    function mk_author_item($args) {

        extract($args);

        $li = '';
        $opt = intval($nav_item[OPT]);
        $idn = intval($nav_item[IDN]);
        $nme = $nav_item[NME];

        $userdata = get_userdata($idn);
        $user_class = NAVT::get_userlevel_class($idn);
        $num_user_posts = get_usernumposts($idn);
        $item_class .= ' ' . 'user-' . $userdata->user_login . ' ' . $user_class . ' ' . TAB_AUTHOR;
        $args['img_class'] = $userdata->user_login . '-img';
        $args['link_url'] = get_author_link(false, $idn);
        $args['anchor_class'] = 'navt_alink';

        if( $opt & SHOW_IF_EMPTY || $num_user_posts > 0 ) {

            if( $opt & GRAPHIC_LINK ) {
                $args['anchor_class'] = 'navt_glink';
                if( $opt & SHOW_AVATAR ) {
                    $args['img_src'] = $nav_item[EXT];
                }
                $li = NAVT::mk_graphic_link($args);
            }
            else {
                extract($args);
                $post_count = (($opt & APPEND_POST_COUNT) ?
                sprintf("<span %s>(%s)</span>", set_klass($is_nostyle, "class='post_count'"), $num_user_posts) : '');

                $meta = array();
                $userinfo = '';

                if( $opt & INC_EMAIL && (!($opt & NO_LINK_TEXT)) ) {
                    if( '' != $userdata->user_email ) {
                        $title = sprintf(__("Send comments to %s"), $alt_text);
                        $meta['email'] = sprintf("<a %s href='mailto:%s' title='%s'>%s</a>\n",
                        set_klass($is_nostyle, "class='navt_email'"), $userdata->user_email, $title, 'email');
                    }
                }

                if( $opt & INC_BIO && (!($opt & NO_LINK_TEXT)) ) {
                    if( '' != $userdata->description ) {
                        $meta['bio'] = sprintf("<p %s>%s</p>\n", set_klass($is_nostyle, "class='navt_bio'"),
                        $userdata->user_description);
                    }
                }

                if( $opt & INC_WEBSITE && (!($opt & NO_LINK_TEXT)) )  {
                    if( '' != $userdata->user_url ) {
                        $title = sprintf(__("Visit %s's web site"), $alt_text);
                        $t = sprintf(__("Web Site"));
                        $meta['web'] = sprintf("<a %s href='%s' title='%s' rel='external'>%s</a>\n",
                        set_klass($is_nostyle, "class='navt_weburl'"),
                        $userdata->user_url, $title, $t);
                    }
                }

                if( count($meta) > 0 ) {
                    $userinfo = sprintf("%s<div %s'>\n", _indentt($in+1), set_klass($is_nostyle, "class='navt_user_details'"));

                    foreach( $meta as $key => $option ) {
                        $userinfo .= sprintf("%s%s", _indentt($in+2), $option);
                    }
                    $userinfo .= sprintf("%s</div>\n", _indentt($in+1));
                }

                if( $opt & SHOW_AVATAR ) {
                    /*
                    * Output:
                    *
                    * <li class="$item_class">
                    *   <a class='navt_alink' href='$author_link' title='View all entries by $nme'>
                    *       <img class='navt_avatar' src='$avatar_src' alt='$nme' />
                    *   </a>
                    *   <div class="navt_user_details">
                    *       [<a class='navt_email' title='mail comments to + $nme' href='mailto: email'>$nme [$post_count]</a>]
                    *       [<p class='navt_bio'>$bio</p>]
                    *       [<a class='navt_weburl' title='$web' href='$web'>$web name</a>]
                    *   </div>
                    * </li>
                    */
                    $title = sprintf(__("View all entries by %s"), $nme);
                    $li  = sprintf("%s<%s %s>\n", _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"));
                    $li .= sprintf("%s<a %s href='%s' title='%s' %s>\n", _indentt($in+1),
                    set_klass($is_nostyle, "class='$anchor_class'"), $link_url, $title, $tags);
                    $li .= sprintf("%s<img %s src='%s' alt='%s' />%s %s\n%s</a>\n", _indentt($in+2),
                    set_klass($is_nostyle, "class='navt_avatar'"),
                    $usr_defined, $alt_text, (!($opt & NO_LINK_TEXT) ? $nme: ''), $post_count, _indentt($in+1));
                    $li .= ('' != $userinfo) ? $userinfo: '';
                    $li .= sprintf("%s%s\n", _indentt($in), $sAfterItem);
                }
                else {
                    /* Plain text */
                    $li  = sprintf("%s<%s %s>\n", _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"));

                    $li .= sprintf("%s<a %s href='%s' title='%s'%s>%s %s</a>\n", _indentt($in+1),
                    set_klass($is_nostyle, "class='$anchor_class'"), $link_url, $alt_text, $tags,
                    (!($opt & NO_LINK_TEXT) ? $nme: ''), $post_count);

                    $li .= ('' != $userinfo) ? $userinfo: '';
                    $li .= sprintf("%s%s\n", _indentt($in), $sAfterItem);
                }
            }
        }
        return( $li );
    }

    /**
     * Create a uri item for the navigation group
     *
     * @param array   $nav_item
     * @param string  $item_class
     * @param string  $sBeforeItem
     * @param string  $sAfterItem
     * @param integer $in
     * @return  HTML string
     */
    function mk_uri_item($args) {

        extract($args);

        $li = '';
        $opt = intval($nav_item[OPT]);
        $nme = $nav_item[NME];
        $ttl = $nav_item[TTL];

        $nameAsClass = sanitize_title_with_dashes(strtolower($nme));
        $args['item_class'] .= ' ' . $nameAsClass . '-uri ' . TAB_ELINK;
        $args['img_class'] = $nameAsClass . '-img';
        $args['tags'] =  ($opt & RELFOLLOW) ? '' : " rel='nofollow' ";
        $args['tags'] .= ($opt & OPEN_SAMEWIN) ? '' : "target='_blank'";
        $args['anchor_class'] = 'navt_elink';
        $args['link_url'] = $ttl;

        if( $opt & GRAPHIC_LINK ) {
            $args['anchor_class'] = 'navt_glink';
            $li = NAVT::mk_graphic_link($args);
        }
        else {
            // Link text with a side graphic
            if( $opt & TEXT_WITH_SIDE_GRAPHIC ) {
                $li = NAVT::mk_text_with_side_graphic($args);
            }
            else if( $opt & TEXT_OVER_GRAPHIC ) {
                $li = NAVT::mk_text_over_graphic($args);
            }
            else {

                /* Plain Text
                * Output:
                *
                *  <li class="$item_class"><a class="navt_elink" href="$ttl" title="$nme">$nme</a></li>
                */
                extract($args);  // get these again
                $li = sprintf("%s<%s %s><a %s href='%s' title='%s' %s>%s</a>%s\n",
                _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"),
                set_klass($is_nostyle, "class='$anchor_class'"), $link_url, $alt_text, $tags, $alt_text, $sAfterItem);
            }
        }
        return( $li );
    }

    /**
     * Create a home or login item for the navigation group
     *
     * @param array   $nav_item
     * @param string  $item_class
     * @param string  $sBeforeItem
     * @param string  $sAfterItem
     * @param integer $in
     * @return  HTML string
     */
    function mk_link_item($args) {

        extract($args);

        $li = '';
        $opt = intval($nav_item[OPT]);
        $nme = $nav_item[NME];
        $site_url = get_settings('home');
        $users_can_register = get_settings('users_can_register');
        $using_login_form = 0;
        $split_anchor = '';
        $args['anchor_class'] = 'navt_ilink';

        if( $nav_item[IDN] == HOMEIDN ) {
            $args['item_class'] .= ' ' . TAB_HOME;
            $args['img_class'] = $link_imgclass = 'home-img ';
            $args['link_url'] = $link_url = get_settings('home');
        }
        else if( $nav_item[IDN] == LOGINIDN ) {
            $args['item_class'] .= ' ' . TAB_ADMIN;
            $args['img_class'] = 'admin-img ';
            $args['tooltip'] = 'Site Admin';
            $args['link_url'] = sprintf("%s/wp-admin/", $site_url);

            if( !is_user_logged_in() ) {
                if( $opt & USE_FORM ) {
                    $using_login_form = 1;

                    // Determine after login redirection
                    $redirect = ($opt & REDIRECT_REFER) ? $_SERVER['PHP_SELF'] :
                    ($opt & REDIRECT_URL) ? $nav_item[EX2]: '';
                    if( '' == $redirect ) {
                        $redirect = $site_url;
                    }

                    // Create a login form
                    $li = sprintf("%s<li class='lform'>\n%s<form name='loginform' id='loginform' class='loginform' action='%s/wp-login.php' method='post'>
                    <input type='hidden' name='redirect_to' value='%s' />
                    <fieldset id='loginbox'>
                    <p><label>".__('Username').":<br /><input type='text' name='log' id='user_login' value='' tabindex='1' /></label></p>
                    <p><label>".__('Password').":<br /><input type='password' name='pwd' id='user_pass' value='' tabindex='2' /></label></p>
                    <p class='rememberme'><label><input name='rememberme' type='checkbox' id='rememberme' value='forever' tabindex='3' /> ".__('Remember me')."</label></p>
                    <p><input type='submit' name='submit' id='login-form' value='Login' tabindex='4' /></p>
                    <p class='lost'><a href='%s/wp-login.php?action=lostpassword'>".__('Lost your password?')."</a></p></fieldset>
                    </form>\n", _indentt($in), _indentt($in+1), $site_url, $redirect, $site_url);

                    if( $users_can_register ) {
                        $li .= sprintf("%s<p><a href='%s/wp-register.php' title='Register' rel='nofollow internal'>".__('Register')."</a></p>\n",
                        _indentt($in+1), $site_url);
                    }

                    $li .= sprintf("%s</li>\n", _indentt($in));
                }
                else {
                    $args['tooltip'] = $nme;
                    $args['img_class'] = 'login-img ';
                    $args['link_url'] = sprintf("%s/wp-login.php", $site_url);

                    if( get_settings('users_can_register') ) {
                        $args['img_class'] = 'register-img ';
                        $args['link_url'] = sprintf("%s/wp-register.php", $site_url);
                    }
                }
            }
            else {
                // user is logged in
                if( $opt & USE_FORM ) {
                    $split_anchor = sprintf("&nbsp;&bull;&nbsp;<a href='%s/wp-login.php?action=logout&redirect_to=%s' title='logout'>".__('Sign out')."</a>",
                    $site_url, $site_url);
                }
            }
        }
        else {
            ; // shouldn't get here...
        }

        if( $opt & GRAPHIC_LINK && !$using_login_form ) {
            $args['anchor_class'] = 'navt_glink';
            $li = NAVT::mk_graphic_link($args);
        }
        else {
            // Link text with a side graphic
            if( $opt & TEXT_WITH_SIDE_GRAPHIC && !$using_login_form ) {
                $li = NAVT::mk_text_with_side_graphic($args);
            }
            else if( $opt & TEXT_OVER_GRAPHIC && !$using_login_form ) {
                $li = NAVT::mk_text_over_graphic($args);
            }
            else {

                if( !$using_login_form ) {
                    /* Plain Text
                    * Output:
                    *
                    *  <li class="$item_class"><a class="navt_ilink" href="$link_url" title="$tooltip">$link_text</a></li>
                    */
                    extract($args); // get these again
                    $li = sprintf("%s<%s %s><a %s href='%s' title='%s' %s>%s</a>%s%s\n", _indentt($in), $sBeforeItem,
                    set_klass($is_nostyle, "class='$item_class'"), set_klass($is_nostyle, "class='$anchor_class'"),
                    $link_url, $tooltip, $tags, $tooltip, $split_anchor, $sAfterItem);
                }
            }
        }
        return( $li );
    }

    /**
     * Create a divider item for the navigation group
     *
     * @param array   $nav_item
     * @param string  $item_class
     * @param string  $sBeforeItem
     * @param string  $sAfterItem
     * @param integer $in
     * @return  HTML string
     */
    function mk_divider_item($args) {

        extract($args);

        $li = '';
        $opt = intval($nav_item[OPT]);
        $nme = $nav_item[NME];

        $nameAsClass = strtolower($nme);
        $nameAsClass = sanitize_title_with_dashes($nameAsClass) . '_cls ';
        $item_class .= ' ' . $nameAsClass;

        if( $opt & PLAIN_TEXT_OPTION ) {
            $item_class .= ' ' . TAB_SUBHEAD;
            $li  = sprintf("%s<%s %s>\n", _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"));
            $li .= sprintf("%s<div %s>\n%s<h2>%s</h2>\n%s</div>\n%s%s\n",
            _indentt($in+1), set_klass($is_nostyle, "class='listdiv'"),
            _indentt($in+2), $nme, _indentt($in+1), _indentt($in), $sAfterItem);
        }

        elseif( $opt & HRULE_OPTION ) {
            $item_class .= ' ' . TAB_HRULE;
            $li  = sprintf("%s<%s %s><hr />%s\n", _indentt($in), $sBeforeItem,
            set_klass($is_nostyle, "class='$item_class'"), $sAfterItem);
        }

        else {
            $item_class .= ' ' . TAB_EMPTY;
            $li = sprintf("%s<%s %s>&nbsp;%s\n", _indentt($in), $sBeforeItem,
            set_klass($is_nostyle, "class='$item_class'"), $sAfterItem);
        }

        return( $li );
    }

    /**
     * Creates a graphic link from an item
     * (styled and unstyled)
     *
     * @param array $args
     * @return string (HTML)
     * @since 95.42
     */
    function mk_graphic_link($args) {
        extract($args);

        /*
        * Output:
        *
        *  <li class="$item_class">
        *      <div class="navt_gl">
        *          <a class="navt_glink" href="$permalink" title="$nme">
        *              <img class="$img_class" src="$blank" alt="$nme" />
        *              <span style='margin-left: -5000px;'>$nme</span>
        *          </a>
        *      </div>
        *  </li>
        */
        $li  = sprintf("%s<%s %s>\n", _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"));
        $li .= sprintf("%s<div %s>\n", _indentt($in+1), set_klass($is_nostyle, "class='navt_gl'"));
        $li .= sprintf("%s<a %s href='%s' title='%s' %s>\n", _indentt($in+2),
        set_klass($is_nostyle,"class='$anchor_class'"), $link_url, $tooltip, $tags);
        $li .= sprintf("%s<img %s src='%s' alt='%s' />\n", _indentt($in+3),
        set_klass($is_nostyle, "class='$img_class'"), $img_src, $nme);
        $li .= sprintf("%s<span style='margin-left:-5000px;'>%s</span>\n", _indentt($in+3), $alt_text);
        $li .= sprintf("%s</a>\n%s</div>\n%s%s\n", _indentt($in+2), _indentt($in+1), _indentt($in), $sAfterItem);

        return($li);
    }

    /**
     * Create text with side graphic link
     *
     * @param array $args
     * @return string (HTML)
     * @since 95.42
     */
    function mk_text_with_side_graphic($args) {
        extract($args);

        /*
        * Output: (with styles)
        *
        *  <li class="$item_class">
        *      <div class="navt_twsg">
        *          <img class="$usr_defined" src="$blank" alt="$nme" />
        *      </div>
        *      <a class="navt_clink" href="$cat_link" title="$tooltip" $tags>Menu Alias[$post_count]</a>
        *  </li>
        */
        $li  = sprintf("%s<%s %s>\n", _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"));
        $li .= sprintf("%s<div %s>\n", _indentt($in+1), set_klass($is_nostyle, "class='navt_twsg'"));
        $li .= sprintf("%s<img %s src='%s' alt='%s' />\n", _indentt($in+2),
        set_klass($is_nostyle, "class='$usr_defined'"), $img_src, $alt_text);
        $li .= sprintf("%s</div>\n%s<a %s href='%s' title='%s' %s>%s%s</a>\n", _indentt($in+1), _indentt($in+1),
        set_klass($is_nostyle, "class='$anchor_class'"), $link_url, $tooltip, $tags, $alt_text, $post_count);
        $li .= sprintf("%s%s\n", _indentt($in), $sAfterItem);

        return($li);
    }

    /**
     * Enter Create text over graphic link
     *
     * @param array $args
     * @return string (HTML)
     * @since 95.42
     */
    function mk_text_over_graphic($args) {
        extract($args);

        /*
        * Output: (with styles)
        *
        *   <li class="$item_class">
        *       <div class='navt_tog $img_class'>
        *           <a class="navt_clink" href="$cat_link" title="$tooltip">Menu Alias[$post_count]</a>
        *       </div>
        *   </li>
        */
        $li  = sprintf("%s<%s %s>\n", _indentt($in), $sBeforeItem, set_klass($is_nostyle, "class='$item_class'"));
        $li .= sprintf("%s<div %s>\n", _indentt($in+1), set_klass($is_nostyle, "class='navt_tog $img_class'"));
        $li .= sprintf("%s<a %s href='%s' title='%s' %s>%s%s</a>\n%s</div>\n%s%s\n", _indentt($in+2),
        set_klass($is_nostyle, "class='$anchor_class'"), $link_url, $tooltip, $tags, $alt_text,
        $post_count, _indentt($in+1), _indentt($in), $sAfterItem);

        return($li);
    }


    /** ---------------------------------------------
     * Builds a dropdown option of a navigation item
     * ----------------------------------------------
     * @param array $nav_item
     * @param integer $in
     * @return returns a HTML formatted <option> </option> string
     */
    function mk_ddoption($nav_item, $in, $is_nostyle) {

        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s\n", __FUNCTION__), $nav_item);

        $dd_option = '';
        global $wp_query;
        $qo = $wp_query->get_queried_object();
        $loc = (!empty($qo->cat_ID) ? (TYPE_CAT . $qo->cat_ID) : (!empty($qo->ID) ? (TYPE_PAGE . $qo->ID) : (TYPE_LINK . HOMEIDN)));

        $can_show = 0;
        $idn = $nav_item[IDN];
        $opt = intval($nav_item[OPT]);
        $typ = $nav_item[TYP];
        $ttl = $nav_item[TTL];
        $nme = $nav_item[NME];

        $is_private = ($opt & ISPRIVATE) ? 1: 0;
        $is_draft = ($opt & ISDRAFTPAGE) ? 1: 0;

        if( $is_private ) {
            $can_show = is_user_logged_in() ? 1: 0;
        }
        else {
            $can_show = ($is_draft && $typ == TYPE_PAGE) ? 0: 1;
        }

        if( $can_show) {

            $is_selected = (($loc == $typ.$idn) ? ' selected="selected"': '');

            switch( $typ ) {

                case TYPE_CAT: {
                    $cat = get_category($idn);
                    $num_posts = $cat->category_count;
                    $can_show = ($opt & SHOW_IF_EMPTY || $num_posts > 0) ? 1: 0;
                    if( $can_show ) {
                        $post_count = ( $opt & APPEND_POST_COUNT ) ? sprintf(" (%s)", $num_posts) : '';
                        $dd_option = sprintf("%s<option%s value=\"%s\">%s %s </option>\n",
                        _indentt($in), $is_selected, get_category_link($idn), $nme, $post_count);
                    }
                    break;
                }
                case TYPE_PAGE: {
                    $dd_option = sprintf("%s<option%s value=\"%s\">%s</option>\n",
                    _indentt($in), $is_selected, get_permalink($idn), $nme);
                    break;
                }
                case TYPE_LINK: {
                    switch( $idn ) {
                        case HOMEIDN: {
                            $dd_option = sprintf("%s<option%s value=\"%s/\">%s</option>\n",
                            _indentt($in), $is_selected, get_bloginfo('home'), $nme);
                            break;
                        }
                        case LOGINIDN: {
                            $link_text = __('Site Admin');
                            $link_url = '/wp-admin/';

                            if( !is_user_logged_in() ) {
                                $link_text = $nme;
                                $link_url = '/wp-login.php';
                                $filter = 'loginout';

                                if( get_settings('users_can_register') ) {
                                    $link_url = '/wp-register.php';
                                    $filter = 'register';
                                }
                            }
                            $href = sprintf("<a href=\"%s\" title=\"%s\">%s</a>", $link_url, $link_text, $link_text);
                            if(isset($filter)) $href = apply_filters($filter, $href);
                            $dd_option = sprintf("%s<option%s value=\"%s\">%s</option>\n",
                            _indentt($in), $is_selected, $link_url, $link_text);
                            break;
                        }
                        break;
                    }
                    break;
                }
                case TYPE_SEP: {
                    if( $opt & PLAIN_TEXT_OPTION ) {
                        $dd_option = sprintf("%s<div %s><h2>%s</h2></div>\n", _indentt($in-2),
                        set_klass($is_nostyle, "class='listdiv'"), $nme);
                    }
                    elseif( $opt & HRULE_OPTION ) {
                        $dd_option = sprintf("%s<div %s><hr /></div>\n", _indentt($in-2),
                        set_klass($is_nostyle, "class='listdiv'"));
                    }
                    else {
                        $dd_option = sprintf("%s<div %s>&nbsp;</div>\n", _indentt($in-2),
                        set_klass($is_nostyle, "class='listdiv'"));
                    }
                    break;
                }
                case TYPE_ELINK: {
                    $dd_option = sprintf("%s<option%s value=\"%s\">%s</option>\n", _indentt($in),
                    $is_selected, $ttl, $nme);;
                    break;
                }
            }// end switch item type
        }

        return($dd_option);
    }

    /**
     * Creates a form for use with dropdown menus
     *
     * @param string $sGroupName
     * @param integer $num
     * @param integer $in
     * @return returns a HTML <form> statement
     */
    function mk_form($sGroupName, $num, $in, $is_nostyle) {

        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s\n", __FUNCTION__));
        $sGroupName = strtolower($sGroupName);

        $form_id = $sGroupName . '_' . $num;
        $html = sprintf("%s<form id='%s_form' name='%s_form'>\n", _indentt($in), $form_id, $form_id);
        $html .= sprintf("%s<select id='%s' name='%s' %s ", _indentt($in+1),
            $form_id, $form_id, set_klass($is_nostyle, "class='dropdown_menu'"));
        $html .= sprintf("onChange='location.href = this.form.%s.options[this.form.%s.selectedIndex].value; return false;'>\n",
            $form_id, $form_id);
        return($html);
    }

    /**
     * Ends a select and form
     *
     * @param integer $in
     * @return returns a HTML </form> statement
     */
    function end_form($in) {
        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf("%s\n", __FUNCTION__));
        $html  = _indentt($in+1) . "</select>\n";
        $html .= _indentt($in) . "</form>\n";
        return($html);
    }

    /**
     * Returns a number representing the hierarchy of a page
     *
     * @param integer $id
     * @param integer $level - hierarchy value (0 = top level)
     * @return integer - hierarchy level
     */
    function get_page_level($id, $level=0) {

        $this_level = $level;
        $pg = get_page($id);
        if( $pg->post_parent != 0 ) {
            $this_level++;
            $this_level = NAVT::get_page_level($pg->post_parent, $this_level);
        }
        return($this_level);
    }

    /**
     * Returns a number representing the hierarchy of a category
     *
     * @param integer $id - category id
     * @param integer $level - hierarchy value (0 = top level)
     * @return integer - hierarchy level
     */
    function get_category_level($id, $level=0) {

        $this_level = $level;
        $cat = get_category($id);
        if( $cat->category_parent != 0) {
            $this_level++;
            $this_level = NAVT::get_category_level($cat->category_parent, $this_level);
        }
        return($this_level);
    }

    /**
     * Routine that does some rudimentatry checking of a user avatar
     *
     * @param string  - avatar url
     * @return boolean - 1 = url meets requirement, 0 - bad url or bad image file
     */
    function check_avatar($avatar_url) {

        // Parse the given url
        $arr = parse_url($avatar_url);
        $hasCorrectMime = 0;

        if( !empty($arr['path']) ) {
            $pi = pathinfo($arr['path']);
            if( !empty($pi['basename']) && !empty($pi['extension'])) {
                $ft = wp_check_filetype(trim($pi['basename']));
                if(!empty($ft['type'])) {
                    $hasCorrectMime = (strstr($ft['type'], 'image') != false) ? 1: 0;
                }
            }
        }
        return($hasCorrectMime);
    }

    /**
     * Returns a css class related to a user's wp role
     *
     * @param integer $user_id
     * @return string - css class
     */
    function get_userlevel_class($user_id) {
        global $wpdb;
        $level = intval(get_usermeta($user_id, $wpdb->prefix . 'user_level'));
        if( $level > 8 ) $class = 'administrator';
        elseif( $level > 4 && $level < 8 ) $class = 'editor';
        elseif( $level > 1 && $level < 5 ) $class = 'author';
        elseif( $level == 1 ) 'contributor';
        else $class = 'subscriber';
        return($class);
    }

    /**
     * Builds a HTML select containing the available avatars
     */
    function build_avatar_list($options_only=1, $set_to_disabled=0, $id='avatars', $picked='', $in=0) {

        $html = '';
        $url = NAVT::get_url();
        $path =  NAVT_PLUGINPATH . AVATAR_IMAGES . '/';
        $files = NAVT::files_scan($path, array('png','gif','jpg','jpeg','bmp'), 1, false);
        $state = $set_to_disabled ? 'disabled="disabled"':'';

        if( count($files) > 0 ) {
            if(!$options_only) {
                $html = sprintf("%s<select id='%s' class='%s' %s>\n", _indentt($in), $id, CLS_AV_SELECT, $state);
            }
            foreach($files as $k => $filename ) {
                $pi = pathinfo($filename);
                $name = $pi['basename'];
                $file_url = $url . AVATAR_IMAGES . '/' . $name;
                $select = ($picked == $file_url) ? 'selected="selected"':'';
                $html .= sprintf("%s<option value=\"%s\"%s>%s</option>\n", _indentt($in+1), $file_url, $select, $name);
            }
            if( !$options_only ) {
                $html .= sprintf("%s</select>\n", _indentt($in));
            }
        }
        return($html);
    }

    /**
     * Scan for files to be included in th
     *
     * @param string $path  - where to begin the search
     * @param string $ext - type of file to search for, false == everything
     * @param integer $depth - depth of search
     * @param boolean $relative - true = scan is relative
     * @return array $files - array of files
     */
    function files_scan($path, $ext = false, $depth = 1, $relative = true) {
        $files = array();

        // Scan for all matching files
        NAVT::_files_scan($path, '', $ext, $depth, $relative, $files);
        return $files;
    }

    /**
     * Returns an array of filenames scanned in a directory
     *
     * @param string $base_path  - where to begin the search
     * @param string $path - directory path
     * @param string $ext - type of file to search for
     * @param integer $depth - depth of search
     * @param boolean $relative - true = scan is relative
     * @param  array $files - array of files
     */
    function _files_scan($base_path, $path, $ext, $depth, $relative, &$files) {
        if (!empty($ext)) {
            if (!is_array($ext)) {
                $ext = array($ext);
            }
            $ext_match = implode('|', $ext);
        }

        // Open the directory
        if(($dir = @dir($base_path . $path)) !== false) {
            // Get all the files
            while(($file = $dir->read()) !== false) {
                // Construct an absolute & relative file path
                $file_path = $path . $file;
                $file_full_path = $base_path . $file_path;

                // If this is a directory, and the depth of scan is greater than 1 then scan it
                if(is_dir($file_full_path) and $depth > 1 and !($file == '.' or $file == '..')) {
                    NAVT::_files_scan($base_path, $file_path . '/', $ext, $depth - 1, $relative, $files);

                    // If this is a matching file then add it to the list
                } elseif(is_file($file_full_path) and (empty($ext) or preg_match('/\.(' . $ext_match . ')$/i', $file))) {
                    $files[] = $relative ? $file_path : $file_full_path;
                }
            }

            // Close the directory
            $dir->close();
        }
    }

    function wp_redirect_wpcb($location, $status) {
        if(NAVTdbg()) navt_write_debug(NAVT_WPHOOKS, sprintf("redirect: %s, status: %s", $location, $status));

        // TEMPORARY WP BUG FIX FOR SCREW'D UP REDIRECTS
        $new_location = str_replace('_php&', '.php?', $location);
        return($new_location);
    }

    /**
     * Backs up the current map and saves an XML file to
     * the local drive
     *
     * @param string $filename
     * @param string $mime_type
     * @since 95.42
     */
    function backup() {

        $ar = NAVT::load_map();
        $i = 1;
        if( count($ar) > 0 ) {
            // Set the output type
            $filename = 'navt_plugin.' . date('Y-m-d') . '.xml';
            header('Content-Description: File Transfer');
            header("Content-Disposition: attachment; filename=$filename");
            header('Content-type: text/xml; charset=' . get_option('blog_charset'), true);
            print '<?xml version="1.0" encoding="' . get_bloginfo('charset') . '"?' . ">\n";?>
            <!--
            generator="navt/<?php echo SCRIPTVERS;?>"
            created="<?php echo date('Y-m-d H:i'); ?>"
            This file contains the backup information for the Navigation Tool For Word Press Plugin (NAVT)
            This backup can be restored using the plugin restore facility in the NAVT List sidebar
            Copyright (c) 2006-2007 Greg Bellucci, The MIT License.
            -->
            <?php
            print "\n<navt major_version='". MAJVER . "' minor_version='" . MINVER . "'>\n";
            foreach( $ar as $itm ) {
                $opt = intval($itm[OPT]);
                $lvl = intval($itm[LVL]);
                $itm[OPT] = (($opt > 0) ? $opt: 0);
                $itm[LVL] = (($lvl > 0) ? $lvl: 0);

                printf("\t<!-- group: %s, item: %03d -->\n", $itm[GRP], $i++);
                print "\t<item>\n";
                print "\t\t" . NAVT::create_xml_entry('grp', $itm[GRP], false) . "\n";
                print "\t\t" . NAVT::create_xml_entry('typ', $itm[TYP], false) . "\n";
                print "\t\t" . NAVT::create_xml_entry('idn', $itm[IDN], false) . "\n";
                print "\t\t" . NAVT::create_xml_entry('ttl', $itm[TTL], true)  . "\n";
                print "\t\t" . NAVT::create_xml_entry('nme', $itm[NME], true)  . "\n";
                print "\t\t" . NAVT::create_xml_entry('opt', $itm[OPT], false) . "\n";
                print "\t\t" . NAVT::create_xml_entry('lvl', $itm[LVL], false) . "\n";
                print "\t\t" . NAVT::create_xml_entry('ext', $itm[EXT], true)  . "\n";
                print "\t\t" . NAVT::create_xml_entry('ex2', $itm[EX2], true)  . "\n";
                print "\t</item>\n";
            }
            print "</navt>\n";
        }
        die(); // prevents any new headers from going out
    }

    /**
     * Create a n XML entry for the backup
     *
     * @param string $token
     * @param string $str
     * @param boolean $encode
     * @return string - XML tag
     */
    function create_xml_entry($token, $str, $encode) {

        $ret = '<' . $token;

        if('' == $str || ' ' == $str ) {
            $ret .= '/>';
        }
        else {
            $ret .= '>';
            $ret .= ( $encode == true ) ? NAVT::cdata($str) : $str;
            $ret .= '</' . $token . '>';
        }

        return($ret);
    }

    /**
     * Encapsulates a string in a CDATA line
     *
     * @param string $str
     * @return encapsulated string
     */
    function cdata($str) {
        $d = "<![CDATA[$str]]>";
        return $d;
    }

}// end class NAVT

/**
 * Format a navt menu array for a navigation item
 *
 * @param string group name
 * @param string page/category/link/whatever type
 * @param string page or category database id
 * @param string page or category title (from database)
 * @param string page or category menu alias (the name that appears in the menu)
 * @param string options
 * @param string extra options
 * @param integer item level (hierarchy)
 * @return array
 */
function navt_format_menu_item($args) {
    extract($args);
    return(array(
    GRP => $grp,
    TYP => $typ,
    IDN => $idn,
    TTL => $ttl,
    NME => $nme,
    OPT => $opt,
    EXT => $ext,
    LVL => $lvl,
    EX2 => $ex2));
}

/**
 * Test for an empty string
 *
 * @param string $string
 * @return boolean 1 = blank, 0 = not blank
 */
function isBlank($string) {

    $ret = 1; // assume it is blank

    if ($string != '') {
        for ($i = 0; $i < strlen($string); $i++) {
            $c = substr($string, $i, 1);
            if ( ($c != "\r" ) && ($c != " ") && ($c != "\n") && ($c != "\t")) {
                $ret = 0;
            }
        }
        return($ret);
    }
}
/**
 * Returns a string of spaces for HTML indentation
 *
 * @param integer $howmany_tabs- the number of indentation tabs to generate
 * @return string the indentation string
 */
function _indentt($howmany_tabs, $tabstring='    ')
{
    $o = '';
    for($x = 0; $x < $howmany_tabs; $x++ )	$o .= $tabstring;
    return($o);
}

/**
 * Returns a comma delineated string containing the names
 * of all the menu groups.
 */
function navt_getAllLists() {

    $gp = array();
    $map_array = NAVT::load_map();
    $grps = NAVT::get_groups($map_array);

    foreach($grps as $g){
        if($g != ID_DEFAULT_GROUP) $gp[] = $g;
    }
    $out = ( count($gp) != 0 ) ? implode(',' , $gp) : null;
    return($out);
}

function navt_output_url() {
    echo(NAVT::get_url());
}

@include('navt-extensions.php');
add_action('init', array('NAVT', 'init'));
?>